#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.1"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"


from mysqldbPool import mysqldbPool
from queryMarkerDb import QueryMarkerDb

class InsertMarkerDb(QueryMarkerDb):

    @staticmethod
    def init_db():
      """
      This function checks if the require type is in the db or now, if not, add them
      """
      # Initialise the snp_type
      snp_types=['primary', 'secondary', 'primary-repeat']
      for snp_type in snp_types:
        # this insert is before building the snp_cache so need to query the db 
        if (QueryMarkerDb.get_snp_type_id_from_db(snp_type) == None):
          InsertMarkerDb._sql.statement("""
              INSERT INTO `T_SNP_Type` (`Name`)
              VALUES (%s);""", (snp_type))

      # Initialise the upload_type
      upload_types=['ref', 'vcf']
      for upload_type in upload_types:
        if (QueryMarkerDb.get_upload_type_id(upload_type) == None):
          InsertMarkerDb._sql.statement("""
              INSERT INTO `T_Upload_Type` (`Upload_Type_Name`)
              VALUES (%s);""", (upload_type))

      # Initialise the job_status
      job_statuss=['stop', 'queue', 'pause', 'run', 'done', 'error', 'check_new_snp_done', 'check_new_snp_error', 'mapper_run', 'mapper_done', 'mapper_error']
      for job_status in job_statuss:
        if (QueryMarkerDb.get_job_status_id(job_status) == None):
          InsertMarkerDb._sql.statement("""
              INSERT INTO `T_Job_Status` (`Job_Status_Name`)
              VALUES (%s);""", (job_status))


    @staticmethod
    def add_species(species_name, species_abbrevation):
        """
        @param species_name (str): species name, to be stored in `T_Species_Name`.`Species_Name`
        @param species_abbrevation (str): species abbrevation, to be stored in `T_Species_Name`.`Species_Abbreviation`
        @return species_id (int): `T_Species_Name`.`ID_Species`
        """
        InsertMarkerDb._sql.statement("""
                       INSERT IGNORE INTO `T_Species_Name`
                       (`Species_Name`, `Species_Abbreviation`)
                       VALUES (%s, %s);
                       """, (species_name, species_abbrevation))
        species_id = InsertMarkerDb._get_species_id_for_name(species_name)[0][0]
        return species_id

    @staticmethod
    def add_snp_name(prefix, snp_name, remark):
        """
        @param snp_name (str): snp_id from the VCF line, to be inserted in the db, make sure this one is unique
                               so before uploading don't use for example chr01, but bnapus_chr01.
                               This snp will be stored in table: `T_SNP_Name`.`SNP_Name`
        @return snp_id (int): `T_SNP_Name`.`ID_SNP`
        """
        if (remark):
          statement = """
                      INSERT INTO `T_SNP_Name`
                      (`PREFIX`, `SNP_Name`, `Remark`)
                      VALUES (%s, %s, %s)
                      """
          values = (prefix, snp_name, remark)
        else:
          statement = """
                      INSERT INTO `T_SNP_Name`
                      (`PREFIX`, `SNP_Name`)
                      VALUES (%s, %s)
                      """
          values = (prefix, snp_name)
        try:
          InsertMarkerDb._sql.statement(statement, values)
        except: # snp already in db
          pass
        snp_id = InsertMarkerDb._get_snp_id_for_name(prefix,snp_name)[0][0]
        return snp_id

    @staticmethod
    def add_snp(snp_id, chrom_id, chrom_pos, snptype):
        """
        @param snp_id (int): `T_SNP_Name`.`ID_SNP`
        @param chrom_pos (int): `T_SNP`.`Chrom_Pos`
        @param chrom_id (int): `T_Chromosome`.`Chromosome_ID`
        @return loc_id (int): `T_SNP`.`Loc_ID`
        """
        statement = """
                    INSERT INTO `T_SNP`
                    (`T_Chromosome_Chromosome_ID`, `Chrom_Pos`, `T_SNP_Name_ID_SNP`, `T_SNP_Type_ID_Type`)
                    VALUES (%s, %s, %s, %s)
                    """
        values = (chrom_id, chrom_pos, snp_id, snptype)
        InsertMarkerDb._sql.statement(statement, values)

        # the snp_loc_id is newly generated by the DB so need to get it from DB and add to cache
        snp_loc_id = InsertMarkerDb.get_snp_loc_id(chrom_pos, chrom_id)
        QueryMarkerDb.add_snp_to_cache(chrom_id, chrom_pos, snp_id, snp_loc_id, snptype)

        return snp_loc_id


    @staticmethod
    def add_reference(ref_name, species_id):
        """
        @param ref_name (str): reference/cultivar name
        @species_id (str): Species id in the db
        @return ref_id (int): 
        """
        InsertMarkerDb._sql.statement("""
                        INSERT IGNORE INTO `T_Reference`
                        (`Reference_Name`, `T_Species_Name_ID_Species`)
                        VALUES (%s, %s)
                        """, (ref_name, species_id))
        ref_id = InsertMarkerDb._get_ref_id_for_ref_name(ref_name)[0][0]
        return ref_id

    @staticmethod
    def add_chromosome(chrom_name, ref_id):
        """
        Add a chromosome to the db.
        @param chrom_name (str): Chromosome name as stored in the database
        @param ref_id (int): reference id in the db (`T_Referenc`.`Reference_ID`)
        @return None
        """
        InsertMarkerDb._sql.statement("""
                        INSERT INTO `T_Chromosome`
                        (`Chromosome_Name`, `T_Reference_Reference_id`)
                        VALUES (%s, %s)
                        """, (chrom_name, ref_id))

    @staticmethod
    def add_cultivar_name(cultivar_name):
        """
        Add a cultivar name to the database
        @param cultivar_name
        @return cultivar_id (int): `T_Cult_Name`.`ID_Cult`
        """
        statement = """
                    INSERT INTO `T_Cult_Name`
                    (`Cult_Name`)
                    VALUES (%s)
                    """
        InsertMarkerDb._sql.statement(statement, (cultivar_name))
        return QueryMarkerDb.is_cult_name_in_db(cultivar_name)

    @staticmethod
    def add_cultivar(id_cult, cult_allele, snp_loc_id):
        """
        @parm id_cult: `T_Cult_Name`.`ID_Cult` to be linked to
        @param cult_allele (str): Cultivar allele for example A/G
        @param snp_loc_id: `T_SNP`.`Loc_ID`
        @return None
        """
        statement = """
                    INSERT INTO `T_Cults`
                    (`Cult_allele`, `T_SNP_Loc_ID`, `T_Cult_Name_ID_Cult`)
                    VALUES (%s, %s, %s)
                    """
        values = (cult_allele, snp_loc_id, id_cult)
        InsertMarkerDb._sql.statement(statement, values)
        QueryMarkerDb.add_cultivar_to_cache(snp_loc_id, id_cult)

    @staticmethod
    def add_cultivar_batch(cultInfoList):
      """
      @parm cultInfoList: is a list of a list of the cultivar information: id_cult, cult_allele, snp_loc_id
      """
      statement = """ INSERT INTO `T_Cults`
                      (`T_Cult_Name_ID_Cult`, `Cult_allele`, `T_SNP_Loc_ID`)
                      VALUES """
      allCultInfoList=[]
      for cultInfo in cultInfoList:
        statement += """(%s, %s, %s),"""
        allCultInfoList.extend(cultInfo)
        QueryMarkerDb.add_cultivar_to_cache(cultInfo[2], cultInfo[0]) # cultInfo[2] is snp_loc_id, [0] is id_cult

      # slice off the comma
      statement = statement[:-1]
      values = tuple(allCultInfoList)
      InsertMarkerDb._sql.statement(statement, values)
   

    @staticmethod
    def insert_job(jobStatusID, spID, refID, uploadDir, uploadType):
        statement = """
                    INSERT INTO `T_Upload_Jobs`
                    (`Job_Status_ID`, `Species_ID`, `Ref_ID`, `Upload_Type_ID`, `Upload_Subdir`)
                    VALUES (%s, %s, %s, %s, %s)
                    """
        values = (jobStatusID, spID, refID, uploadType, uploadDir)
        InsertMarkerDb._sql.statement(statement, values)




if __name__ == "__main__":
  InsertMarkerDb.init_db()

  cultList=[]
  cultList.append([1,'A',201])
  cultList.append([2,'G',301])

  InsertMarkerDb.add_cultivar_batch(cultList)

    # Uncommented, please don't mess arround with input testing while the db
    # settings are the real db no a testdb in the config file, thanks.
    ## print InsertMarkerDb.add_reference("testref3", "8")
