#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.1"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"

import sys
import os
import re

from queryMarkerDb import QueryMarkerDb
from insertMarkerDb import InsertMarkerDb
from process_output import Process_output
from config import config

import logging
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__),'..','log','unisnpdb.log'), level=config['logLevel'], format="%(asctime)s:%(levelname)s:%(message)s")


class Process_fasta_snp:

    def __init__(self, vcf_fn, flanking_region=config['fasta_snp_flanking_size']):
        """
        @param vcf_fn (str): location for the uploaded vcf file
        @param flanking_region (int): flanking region around snp, e.g. 250 will take 250bp upstream AND 250 downstream (500bp total)
        @param min_length (int): Minimum length for flanking fasta, used at start and and for contigs
        """
        self.__vcf_fn = vcf_fn
        self.__flanking_region = flanking_region
        
        
    def __process_vcf_file(self, vcf_fn):
        """
        @param vcf_fn (str): location for the uploaded_vcf_file
        @return pos_dic (dic): key=chrom_name, value=list(snp_name, snp_position(int))
        """
        pos_dic = {}
        vcf_fh = open(vcf_fn, 'r')
        cult_id_list = []
        for line in vcf_fh:
            if not line.startswith("#"):
                tabbed = line.split()
                if tabbed[0] not in pos_dic:
                    pos_dic[tabbed[0]] = []
                try:
                    pos_dic[tabbed[0]].append([tabbed[2],int(tabbed[1]),line])
                except:
                    print >> sys.stderr, 'ERROR in line: %s' %(line)
                    exit()
            elif line.startswith("#CHROM"):
                cult_list = filter(None, re.split("\t+", line)[9:])
                    
                for cult in cult_list:
                    # as all cultivar names are inside the single quote, take them out
                    cult = re.sub("'", "", cult)
                    cult_id = QueryMarkerDb.is_cult_name_in_db(cult)
                    if cult_id == None:
                      cult_id = InsertMarkerDb.add_cultivar_name(cult)
                    cult_id_list.append(cult_id)

        for key in pos_dic:
            # sort on position
            pos_dic[key] = sorted(pos_dic[key], key=lambda row:row[1])

        return pos_dic, cult_id_list


    def __get_sub_seq(self, tmp_seq, offset_list, vcf_list, ref_id, chrom_name, cult_id_list):
        """
        @param tmp_seq    (str): sequence for a chromosome (load it per chromosome instead of complete fasta to avoid memory issues)
        @param offset_seq (list): a list containing tuples of contig offset start and end locations. 
        @param vcf_list   (list): a list containing a list: ((snp, position), )
        """
        minContigLen=self.__flanking_region * 2 + 1

        for snp,position,vcf_line in vcf_list:
            # Get contig start and end position
            (contig_start_pos,contig_end_pos) = self.__get_contig_pos(position, offset_list)
            thisContigLen=contig_end_pos - contig_start_pos + 1
            if thisContigLen < minContigLen:
              # instead of ignore it, load it as a primary snp, mark 'short' in snp_name remark column.
              chrom_id = QueryMarkerDb.get_chrom_id(chrom_name, ref_id)
              Process_output.direct_add_primary(chrom_id, position, vcf_line, cult_id_list, 'Short')
              #logging.warn("snp (%s) is in short (%d) contig, min require %d, ignored!" %(snp, thisContigLen, minContigLen)) 
              continue

            (snp_start_pos, snp_pos, snp_end_pos) = self.__convert_to_contig_pos(contig_start_pos, contig_end_pos, position)

            # the substring index is from 0, so use 1 before the snp position
            output = ">%d_%s\n%s\n" %(snp_pos,snp,tmp_seq[snp_start_pos-1:snp_end_pos])
            self.__output_fh.write(output)


    def __convert_to_contig_pos(self, contig_start, contig_end, snp_pos):
      """
      """
      allowable_left_flank = snp_pos - contig_start
      allowable_right_flank = contig_end - snp_pos

      left_flank = self.__flanking_region
      right_flank = self.__flanking_region

      if allowable_left_flank < self.__flanking_region:
        left_flank = allowable_left_flank
        right_flank += (self.__flanking_region - allowable_left_flank)
      if allowable_right_flank < self.__flanking_region:
        right_flank = allowable_right_flank
        left_flank += (self.__flanking_region - allowable_right_flank)

      left_flank_pos = snp_pos - left_flank
      right_flank_pos = snp_pos + right_flank
      return left_flank_pos, left_flank + 1, right_flank_pos


    def __get_contig_pos(self, position, offset_list):
      """
      @param position    (int): snp position
      @param offset_list (list): a list of tuples containing the contig start and end position
      """
      for (start, end) in offset_list:
        if ((position >= start) and (position <= end)):
          return start, end
      raise Exception("SNP_NOT_IN_OFFSET_FILE")


    def get_flanking_regions(self, output_fn, fasta_ref_fn, ref_id=None):
        """
        @param output_fn (str): filename and location to write the outputfile
        @param fasta_ref_fn (str): location to the fasta reference file
        @param ref_id (int): reference id in the database, default None
                             But only use default when not using with unisnpdb,
                             for example when using this script as cmd line script.
        """
        self.__output_fh = open(output_fn, 'w')
        fasta_ref_fh = open(fasta_ref_fn, 'r')
        
        vcf_list = []
        # collect multiple sequence line into this tmp_seq variable
        tmp_seq = ''
        offset_list=[]
        vcf_dic = {}
        cult_id_list = []

        vcf_dic, cult_id_list = self.__process_vcf_file(self.__vcf_fn)

        # Go through the whole fasta reference file
        fasta_id=''
        for line in fasta_ref_fh:
            line = line.replace('\n', '')
            # collect info of all sequence
            if line.startswith(">"):
                # handle the previously collected sequence
                if tmp_seq != '' and vcf_list != []:
                    # use the offset file of the previous reference
                    offset_list = self.__get_offset_list( offsetFile )
                    # QueryMarkerDb.get_chrom_id('XA01_v3.0', 1)
                    # fasta_id is the chromosome name 
                    self.__get_sub_seq(tmp_seq, offset_list, vcf_list, ref_id, fasta_id, cult_id_list)

                # prepare for the new sequenc
                tmp_seq = ""
                fasta_id = line.replace('>', '')
                # offset file is defined as the reference seq id plus the word '-offset.gff3'
                offsetFile = os.path.join(os.path.dirname(fasta_ref_fn), "%s-offset.gff3" %(fasta_id))

                # take out the added ref_id in the fasta reference id as the fasta_id
                if ref_id != None:
                    fasta_id = re.sub(r"^%d_" %(ref_id), "", fasta_id)
                # logging.debug("fasta_id: "+fasta_id+"   vcf_dic.key: " + ', '.join(vcf_dic.keys()));
                # collect all the required reference fasta sequences based on the fasta_id
                if fasta_id in vcf_dic:
                    vcf_list = vcf_dic[fasta_id]
                else: 
                    vcf_list = []
            else:
                tmp_seq += line
        if vcf_list != []:
            offset_list = self.__get_offset_list( offsetFile )
            self.__get_sub_seq(tmp_seq, offset_list, vcf_list, ref_id, fasta_id, cult_id_list)
        self.__output_fh.close()
                

    def __get_offset_list(self, offsetFile):
      offset_list=[]
      with open(offsetFile) as fh:
        for line in fh:
          if re.match('^#',line) or re.match('^\s*$',line):
            continue
          line_ele=line.split("\t")
          offset_list.append((int(line_ele[3]),int(line_ele[4])))
      logging.debug("offsetFile: " + offsetFile)
      return offset_list
    


def main():
    obj = Process_fasta_snp(vcf_fn)
    # testing using the first reference sequence
    obj.get_flanking_regions(output_fn, fasta_ref_fn, 1)
    

if __name__ == "__main__":
    try: 
        fasta_ref_fn = sys.argv[1]
        vcf_fn = sys.argv[2]
        output_fn = sys.argv[3]
    except IndexError:
        print "usage: %s fasta_ref.fasta vcf_file.vcf outfile.fasta" %(sys.argv[0])
        print "Example: python process_fasta_snp.py ~/testSets/test/ref.fasta ~/testSets/test/Darmor.vcf ~/testSets/test/chrA01_flankSeq.fasta"
        exit()
    main()
