#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.2"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"

# Known bugs:
# I don't think the static value __indexed works while running under apache
# with mod_wsgi.

import re
from config import config
import os
import commands
from insertMarkerDb import InsertMarkerDb
from queryMarkerDb import QueryMarkerDb
from shutil import copyfile
from process_output import Process_output


import logging
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__),'..','log','unisnpdb.log'), level=config['logLevel'], format="%(asctime)s:%(levelname)s:%(message)s")


class Process_input:

    __indexed = True

    def __init__(self, ref_id, species_id):
        """
        The files are stored based on the ref_id and species_id, with
        species_id as directory name
        and ref_id in front of the fasta id, to avoid problems with the same fasta_ids
        @param ref_id (int): reference id linked in db `T_Reference`.`Reference_ID`
        @param species_id (int): Species id in db: `T_Species_Name`.`ID_Species`
        """
        self.__ref_id = ref_id
        self.__species_id = species_id
        if config['workhorse_user'] is None:
          self.__outpath = os.path.join(config['data_dir'], str(species_id)) 
        else:
          self.__outpath = os.path.join(config['workhorse_data_dir'], str(species_id)) 
        self.__ref_db_fn = os.path.join(self.__outpath, config['ref_db_fn'])
        if not os.path.exists(self.__outpath):
            os.makedirs(self.__outpath)
        self.__added_sequences = 0
        
        
    def __get_existing_ref_ids(self):
        """
        @return reference_ids (set): reference ids who are already in the fasta database
        """
        ref_ids = set()
        if os.path.exists(self.__ref_db_fn):
            for line in open(self.__ref_db_fn, 'r'):
                if line.startswith(">"):
                    ref_id = line.replace(">", "").replace("\n", "")
                    ref_ids.add(ref_id)
            return ref_ids
        else:
            fh = open(self.__ref_db_fn, 'w')
            fh.close()
            return set()
        

    
    def add_ref_to_ref_file(self, ref_fn, offset_fn):
        """
        @param ref_fn (str): file location for the uploaded reference file
        """
        ref_ids = self.__get_existing_ref_ids()
        fh = open(self.__ref_db_fn, 'a')
        readmeFH=open(os.path.join(self.__outpath, 'NAMES.txt'), 'a')
        add = True
        for line in open(ref_fn, 'r'):
            if line.startswith(">"):
                chrom_name = line.replace(">", "").replace("\n", "")
                fasta_id = "%d_%s" %(self.__ref_id, chrom_name)
                
                chrom_id = QueryMarkerDb.get_chrom_id(chrom_name, self.__ref_id)
                if chrom_id == None:
                    InsertMarkerDb.add_chromosome(chrom_name, self.__ref_id)

                # only add if not already in the file
                if fasta_id not in ref_ids:
                    add=True
                    Process_input.__indexed = False
                    self.__added_sequences += 1
                    fh.write(">%s\n" %(fasta_id))

                    idOffsetFile = "%s-offset.gff3" %(fasta_id)
                    copyfile(offset_fn, os.path.join(self.__outpath, idOffsetFile))

                    nameOffsetFile = "%s_%s-offset.gff3" %(QueryMarkerDb.get_ref_name(self.__ref_id), chrom_name)
                    readmeFH.write( idOffsetFile + "\t" + nameOffsetFile + "\n")

                else:
                    add = False
            elif add:
                fh.write(line)
        fh.close()
        readmeFH.close()


    @staticmethod
    def __run_system_cmd(cmd):
        """
        run system command, be careful with this and don't run commands with user input as parameter
        @param cmd (str): system command to run
        @raise Exception when cmd failed
        """
        logging.info(cmd)
        status, result = commands.getstatusoutput(cmd)
        if status != 0:
            raise Exception, "Failed to run cmd %s with exit code %d!" %(cmd, status)

    def index_db_ref_file(self):
        """
        Index db, only when not indexed
        """
        if not Process_input.is_indexed():
            index_cmd = 'makeblastdb -in %s -dbtype nucl' %(self.__ref_db_fn)
            #if config['workhorse_server'] == 'localhost':
            Process_input.__run_system_cmd("'%s'" %(index_cmd))
            #else:
            #  Process_input.__run_system_cmd("ssh %s@%s '%s'" %(config['workhorse_user'], config['workhorse_server'], index_cmd))
            # command: makeblastdb -in self.__ref_db_fn -dbtype nucl
            Process_input.__indexed = True

    def run_mapper(self, flanking_fn, out_blast):
        """
        @param flanking_fn (str): flanking file location
        @param out_blast (str): location to save the blast file
        """
        if not Process_input.is_indexed():
            self.index_db_ref_file()

        blast_cmd = 'blastn -query %s -db %s -out %s -outfmt "6 std qlen qseq sseq"' %(flanking_fn, self.__ref_db_fn, out_blast)
        if config['n_blast_threads']:
          blast_cmd += " -num_threads %d" %(config['n_blast_threads'])
        if config['evalue']:
          blast_cmd += " -evalue %s" %(config['evalue'])

        #if config['workhorse_server'] == 'localhost':
        Process_input.__run_system_cmd(blast_cmd)
        #else:
        #  system_cmd = "ssh %s@%s '%s'" %(config['workhorse_user'], config['workhorse_server'], blast_cmd)
        #  Process_input.__run_system_cmd(system_cmd)
        logging.info("Finished blasting")


    @staticmethod
    def is_indexed():
        """
        check if database is indexed
        @return bool
        """
        return Process_input.__indexed
        
        
    @staticmethod
    def extract_new_snp_in_vcf(ref_id, org_vcf_fn):
      new_vcf_fn = os.path.join(os.path.dirname(org_vcf_fn), 'new_snp_file.vcf')
      org_vcf_fh = open(org_vcf_fn, 'r')
      new_vcf_fh = open(new_vcf_fn, 'w')

      cult_id_list = []

      primaryTypeID = QueryMarkerDb.get_snp_type_id('primary')
      secondTypeID = QueryMarkerDb.get_snp_type_id('secondary')
      for line in org_vcf_fh:
        line = line.replace("\n", "")
        if not line.startswith('#'):
          # handles the snp line
          # extract chrom name and position
          tabbed = line.split("\t")
          chrom_name = tabbed[0]
          chrom_pos = int(tabbed[1])

          chrom_id = QueryMarkerDb.get_chrom_id(chrom_name, ref_id)
          snp_info = QueryMarkerDb.get_snp_in_chr( chrom_id, chrom_pos)
          if snp_info[0] is None:
            # not exist in db, it's new snp, add to new_vcf_fh
            new_vcf_fh.write(line + "\n")
          else:
            # already in db, update the existing snp
            snp_loc_id = snp_info[1]
            snp_type_id = snp_info[2]
            if (snp_type_id == secondTypeID):
              # update its type to to primary
              QueryMarkerDb.update_snp_type(snp_loc_id, primaryTypeID) 
            # add genotype for 2nd snp, update cultivars for primary and primary-repeat
            Process_output.upload_cultivar(snp_loc_id, line, cult_id_list)
        elif line.startswith('#CHROM'):
          new_vcf_fh.write(line + "\n")
          # extract the cultivar names
          cult_list = filter(None, re.split("\t+", line)[9:])
          for cult in cult_list:
            cult = re.sub("'", "", cult)
            cult_id = QueryMarkerDb.is_cult_name_in_db(cult)
            if cult_id == None:
              #print ("process_input - insert cult:**%s**" %(cult))
              cult_id = InsertMarkerDb.add_cultivar_name(cult)
            cult_id_list.append(cult_id)
        else:
          new_vcf_fh.write(line + "\n")

      return new_vcf_fn
            

if __name__ == "__main__":
    #blast_hits = Process_output.parse_blast("/home/s4326000/workspace/markerdb/data/test/2/test01.blast")
    #Process_output.parse_vcf_after_blast(6, blast_hits, '/home/s4326000/workspace/markerdb/data/test/2/test01.vcf')
    #vcf = Vcf_allele_handler('G', 'A', '0|1:7', "GT:DP")
    #print vcf.get_alleles()

    # Test extract_new_snp_in_vcf()
    extract_new_snp_in_vcf(1, '/home/uqccha18/testData/test-extract_new_snp_in_vcf.vcf')
    
    










