#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.1"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"

import re
import os

from queryMarkerDb import QueryMarkerDb
from insertMarkerDb import InsertMarkerDb
from vcf_allele_handler import Vcf_allele_handler
from config import config

import logging
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__),'..','log','unisnpdb.log'), level=config['logLevel'], format="%(asctime)s:%(levelname)s:%(message)s")


class Process_output:

    @staticmethod
    def numbers_to_int_float(value):
        """
        usage: map(Process_output.numbers_to_int_float, list)
        Just a helper to convert all items in a list with strings
        to a float (if float) or int (if int) else it will be kept as
        a string.        
        """
        value=value.strip()
        if value.isdigit():
            return int(value)
        elif re.match("^\d+?\.\d+?$", value):
            return float(value)
        else:
            return value
    
    @staticmethod
    def parse_blast(blast_file, min_bit_score=config['min_bit_score']):
        """
        Assuming only the snps NOT in db is blast, so all primary snp hits will get a new id
        @param blast_file: blast tabular file location
                           use option: "-outfmt '6 std qlen qseq sseq'"
        @return: repeat_hits (array): key: snp_id, 
        @return: snd_hits (dic): key: snp_id, value: list with all blast_hits (dic)
        """
        # output expected from cmd:
        # blastn -query test_snps.fasta -db ref_file.fasta -out out.blast -outfmt '6 std qlen qseq sseq'

        blast_hits = {}

        blast_fh = open(blast_file)

        for line in blast_fh:
            line = line.replace("\n", "")



            ##### This part can be put to blast parsing in parallel manner
            #####   Can use a much simpler file format. This function can just read in the file.
            ##### CHANGE

            # query_id should have the local snp position attached in the front of the query_id, eg. 251_<flank_seq_id>
            query_id, subject_id, perc_identity, alignment_length,\
                mismatches, gap_opens, q_start, q_end, s_start, s_end,\
                evalue, bit_score, qlen, qseq, sseq  = map(Process_output.numbers_to_int_float, line.split("\t"))

            if bit_score >= float(min_bit_score):             
                snpLocalPos, snp_id = query_id.split("_", 1)
                ref_id, chrom = subject_id.split("_", 1)

                # get the snp position in the reference
                snp_position = Process_output.get_snp_ref_position(qseq, sseq, q_start, s_start, snpLocalPos)

                chrom_id = QueryMarkerDb.get_chrom_id(chrom, ref_id)
                # the snp_position contains gap info if hitting a gap, thus snp_position needs to be a string
                result = {"ref_id":int(ref_id),"chrom":chrom,"snp_position":snp_position,\
                          "bit_score":bit_score, "chrom_id":chrom_id}

                # keep the qseq and sseq for the gap alignment examination
                try:
                  snp_position_int = int(snp_position)
                except ValueError:
                  result.update({"qseq":qseq, "sseq":sseq})

                if snp_id not in blast_hits:
                    blast_hits[snp_id] = []
                blast_hits[snp_id].append(result)

                # ken: for testing
                #Process_output.test_compare_custom_bit_score(line)

            ##### END



        # sort based on highest score first
        for key in blast_hits:
            blast_hits[key] = sorted(blast_hits[key], key=lambda sub_key:sub_key['bit_score'], reverse=True)

        # ken: for testing
        #Process_output.test_repeat_criterion(blast_hits)

        return blast_hits


    @staticmethod
    def test_repeat_criterion(blast_hits):
      hitFH=open('/tmp/numOfHits.tsv','w')
      binFH=open('/tmp/numOfHitsBin.tsv','w')
      numOfHits_list={}
      for key in blast_hits:
        numOfHits=len(blast_hits[key])
        refSubCount={}
        for hit in blast_hits[key]:
          thisID=int(hit['ref_id'])
          if thisID not in refSubCount:
            refSubCount[thisID] = 0
          refSubCount[thisID] += 1
        hitFH.write("%s\t%d" %(key, numOfHits))
        for i in range(1,4):
          if i not in refSubCount:
            refSubCount[i]=0
          hitFH.write("\t%d" %(refSubCount[i]))
        hitFH.write("\n")

        if numOfHits not in numOfHits_list:
          numOfHits_list[numOfHits] = 0
        numOfHits_list[numOfHits] += 1

      for key in numOfHits_list:
        binFH.write("%d\t%d\n" %(key, numOfHits_list[key]))

      hitFH.close()
      binFH.close()


    @staticmethod
    def test_compare_custom_bit_score(blastLine):
      query_id, subject_id, perc_identity, alignment_length,\
          mismatches, gap_opens, q_start, q_end, s_start, s_end,\
          evalue, bit_score, qlen, qseq, sseq  = map(Process_output.numbers_to_int_float, blastLine.split("\t"))
      custom_score = (alignment_length * (perc_identity/100.0))/qlen

      scoreFH=open('/tmp/score.txt','a')
      scoreFH.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" %(query_id, subject_id, bit_score, evalue, mismatches, gap_opens, alignment_length, perc_identity, qlen))
      scoreFH.close()


    @staticmethod
    def get_snp_ref_position(qseq, sseq, q_start, s_start, snpLocalPos):
      globalLen=0     # record the current counted length in subject
      snpLocalPosInt=int(snpLocalPos)
      i=int(q_start)
      # from q_start and walk the number of snpLocalPos of times
      while (i<=snpLocalPosInt):
        globalLen += 1
        localCount=i-q_start
        # when it is a '-' in qseq, increase snpLocalPosInt by one so 'i' will count an extra 1 more
        if (qseq[localCount:localCount+1] == '-'):
          snpLocalPosInt += 1
        i+=1
      # check if the final snp position matched a gap
      if sseq[globalLen-1:globalLen] == '-':
        global_snp_gap_pos = Process_output.__get_gap_info(qseq,s_start, sseq, globalLen) 
        return global_snp_gap_pos
      ref_snp_seq=sseq[0:globalLen]
      global_snp_pos= s_start + globalLen - 1 - ref_snp_seq.count('-')
      return global_snp_pos


    @staticmethod
    def __get_gap_info(qseq, s_start, sseq, sGapLen):
      """
      Handle when snp aligned to a gap in sseq
      """
      local_gap_len = 0  # init local_gap_len for recording the total len of the snp gap
      # travel at the gap and backward
      gap_pointer = sGapLen
      while (sseq[gap_pointer-1:gap_pointer] == '-'):
        local_gap_len += 1
        gap_pointer -= 1

      front_gap_len = local_gap_len   # record the front part of gap size

      # travel at the gap+1 forward
      gap_pointer = sGapLen + 1
      while (sseq[gap_pointer-1:gap_pointer] == '-'):
        local_gap_len += 1
        gap_pointer += 1
      ref_snp_seq = sseq[0:sGapLen]
      global_snp_pos= s_start + sGapLen - 1 - ref_snp_seq.count('-')
      global_snp_gap_pos = "%d_%d_%d_%d" %(global_snp_pos, local_gap_len, front_gap_len, sGapLen)

      return global_snp_gap_pos


            
    @staticmethod
    def __get_org_snp_hit(hits, vcf_chrom_id, vcf_pos):
      """
      This is a faster version of __separate_first_and_second_hits
      """
      for hit in hits:
          if vcf_chrom_id == hit['chrom_id'] and vcf_pos == hit['snp_position']:
            return hit
      return None

            
        
    @staticmethod
    def __separate_first_and_second_hits(hits, vcf_chrom_id, vcf_pos):
        """
        separate first (original hit) and second hits.
        @param vcf_pos (int): position of the snp according to vcf file
        @param vcf_chrom: Chromosome for snp specified in vcf line
        @return: (first_hit, second_hits(list) )
        """
        # make a copy so doesn't change the original data
        second_hits = list(hits)

        for first_hit in second_hits:
          if vcf_chrom_id == first_hit['chrom_id'] and vcf_pos == first_hit['snp_position']:
            second_hits.remove(first_hit)
            return first_hit, second_hits
        return None, second_hits
        

    @staticmethod
    def parse_vcf_after_blast(species_id, ref_id, blast_hits, vcf_file):
        """
        @param blast_hits (dic): results from static method parse_blast
        @param vcf_file (str): location of vcf file
        """
        cult_id_list = []
        vcf_fh = open(vcf_file, 'r')

        # logging.debug("vcf_file: %s" %(vcf_file))

        repeatThreshold = QueryMarkerDb.get_species_copy_number(species_id) * QueryMarkerDb.get_number_of_ref(species_id)
        logging.debug("Using RepeatThreshold = %d"%(repeatThreshold))

        # initialise the exception_record.tsv to record all types of exceptional cases
        exceptionFH = open(os.path.join(os.path.dirname(vcf_file),'exception_record.tsv'), 'w')

        for line in vcf_fh:
            #logging.debug("line in vcf_fh: %s" %(line))
            line = line.replace("\n", "")
            if not line.startswith('#'):
                tabbed = line.split("\t")
                snp_id = tabbed[2]
                vcf_snp_pos = int(tabbed[1])
                vcf_chrom_name = tabbed[0]
                chrom_id = QueryMarkerDb.get_chrom_id(vcf_chrom_name, ref_id)
                if snp_id in blast_hits:



                    ##### This part can be handled in the blast parser which will be handle in parallel 
                    #####   Thus, the blast_hits dictionary tagged repeat
                    #####         and none repeat ones are no-gap hits
                    ##### CHANGE
                    hits_with_gap = blast_hits[snp_id]

                    if len(hits_with_gap) > repeatThreshold:
                      snp_type_label='primary-repeat'
                    else:
                      snp_type_label='primary-secondary'
                    hits_no_gap = Process_output.__extract_gap_hit(hits_with_gap, snp_type_label, exceptionFH)
                    ##### END


                    # return all snp_ids and check if there is any of the hitted location is a primary-repeat
                    (this_snp_ids, is_hit_repeat) = Process_output.__fill_snp_info(hits_no_gap)
                    this_snp_id = Process_output.__get_unique_snp_id(this_snp_ids)
                    if this_snp_id is None:
                      logging.warn("snpPos --%d-- hits multi snp id, snp id with highest bit score is used"%(vcf_snp_pos))
                      # The blast_hits is sorted by bit-score in desc order thus the first one has the highest score
                      this_snp_id = this_snp_ids[0]


                    # if the number of hits > repeatThreshold or any of the hits hitting a primary-repeat, process_primary_repeat
                    #  the criterion use hits with the gap alignment

                    ##### This part can be handled in the blast parser which will be handle in parallel 
                    ##### CHANGE
                    if len(hits_with_gap) > repeatThreshold or is_hit_repeat:
                      org_snp_hit = Process_output.__get_org_snp_hit(hits_no_gap, chrom_id, vcf_snp_pos)
                      Process_output.process_primary_repeat(org_snp_hit, this_snp_id, line, cult_id_list)
                    else:
                    ##### END

                      # else process primary and secondary hits
                      primary_hit, second_hits = Process_output.__separate_first_and_second_hits(hits_no_gap, chrom_id, vcf_snp_pos)

                      # check if the snp flanking seq did not hit itself
                      #  Change for not hitting itself but hitting other secondary position is extremely low, 
                      #  or just impossible if blast does a proper job.
                      #  Just putting this condition here as a safety net.
                      if primary_hit == None:
                        #Process_output.__handle_no_primary_hit(vcf_chrom_name, vcf_snp_pos, this_snp_ids, line, exceptionFH)
                        #logging.error("This vcf line is ignored!")
                        logging.error("Blast did not hit self query sequence but other 2nd snp location. (chrom_name, snp_pos, 2ndSnpID): (%s, %d)"%(vcf_chrom_name, vcf_snp_pos, this_snp_id))
                        continue

                      Process_output.process_primary(primary_hit, this_snp_id, line, cult_id_list)
                      Process_output.process_secondary(second_hits, this_snp_id, line, cult_id_list)

                else:
                  #Process_output.__handle_trivial_flanking_seq(snp_id, line, exceptionFH)
                  Process_output.direct_add_primary(chrom_id, vcf_snp_pos, line, cult_id_list, 'NoHit')
                  continue

            elif line.startswith("#CHROM"):
                cult_list = filter(None, re.split("\t+", line)[9:])
                
                for cult in cult_list:
                    # as all cultivar names are inside the single quote, take them out
                    cult = re.sub("'", "", cult)
                    cult_id = QueryMarkerDb.is_cult_name_in_db(cult)
                    if cult_id == None:
                      cult_id = InsertMarkerDb.add_cultivar_name(cult)
                    cult_id_list.append(cult_id)

        exceptionFH.close()



    @staticmethod
    def direct_add_primary(chrom_id, snp_pos, vcf_line, cult_id_list, remarkTag):
      primaryTypeID = QueryMarkerDb.get_snp_type_id('primary')
      new_snp_nameid = QueryMarkerDb.get_new_snp_nameid()
      this_snp_id = InsertMarkerDb.add_snp_name(config['new_snp_prefix'], new_snp_nameid, remarkTag)
      this_snp_loc_id = InsertMarkerDb.add_snp(this_snp_id, int(chrom_id), snp_pos, primaryTypeID)
      Process_output.upload_cultivar(this_snp_loc_id, vcf_line, cult_id_list)



    @staticmethod
    def process_primary(primary_hit, this_snp_id, vcf_line, cult_id_list):

      # process the primary hit
      # Can not assume that this primary_hit(chrID, chrPos) are not in DB even Process_input.extract_new_snp_in_vcf has been processed before blasting 
      #  As the current run may generate a new 2nd snp at this primary snp location.
      #  thus a position check is needed.
      #  cases (chrID, chrPos):
      #    primary snp: a duplication snp entry in vcf, give warning and update this primary snp cultivar list
      #    secondary snp: change it to primary snp and add cultivar list
      #    primary repeat snp: no change
      primaryTypeID = QueryMarkerDb.get_snp_type_id('primary')

      this_snp_loc_id = primary_hit['snp_loc_id']

      #if 'snp_loc_id' not in primary_hit: 
      if this_snp_loc_id is None: 
        # this primary snp is not in db
        this_snp_loc_id = InsertMarkerDb.add_snp(this_snp_id, int(primary_hit['chrom_id']), int(primary_hit['snp_position']), primaryTypeID)
        Process_output.upload_cultivar(this_snp_loc_id, vcf_line, cult_id_list)
      else:
        primary_snp_type_id = primary_hit['snp_type_id']
        repeatTypeID = QueryMarkerDb.get_snp_type_id('primary-repeat')
        secondTypeID = QueryMarkerDb.get_snp_type_id('secondary')

        if primary_snp_type_id == primaryTypeID:
          # this_snp_id is from the primary_hit, check its snp_type
          Process_output.upload_cultivar(this_snp_loc_id, vcf_line, cult_id_list)
        elif primary_snp_type_id == secondTypeID:
          # when it is secondary snp, update its type to to primary
          QueryMarkerDb.update_snp_type(this_snp_loc_id, primaryTypeID)
          Process_output.upload_cultivar(this_snp_loc_id, vcf_line, cult_id_list)
        elif (primary_snp_type_id == repeatTypeID):
          # the primary-repeat snp_type should not be processed in process_primary
          raise Exception("ERROR: Program error, primary-repeat snp_type should not be processed in process_primary (this_snp_loc_id:%d)"%(this_snp_loc_id))
        else:
          raise Exception("ERROR: Database error, unknown primary_snp_type_id %d for this_snp_loc_id %d"%(primary_snp_type_id, this_snp_loc_id))



    @staticmethod
    def process_secondary(second_hits, this_snp_id, vcf_line, cult_id_list):
      # process all secondary snps
      secondTypeID = QueryMarkerDb.get_snp_type_id('secondary')
      for second_hit in second_hits:

        second_snp_id = second_hit['snp_id']
        if second_snp_id is None: 
          try:
            # Add this 2nd snp if it is not in DB ( second_hit['snp_id'] is not set )
            InsertMarkerDb.add_snp(this_snp_id, int(second_hit['chrom_id']), int(second_hit['snp_position']), secondTypeID)
          except:
            logging.warn("chrom_id %s with snp_position %s was inserted in the same blast hit"%(second_hit['chrom_id'], second_hit['snp_position']))
        else:
          # As __get_all_snp_id and __get_unique_snp_id would make sure this_snp_id is unique
          #  When second_snp_id is not None, it has to be equal to this_snp_id and no need to insert
          if second_snp_id != this_snp_id:
            # The only case this would happen when a snp hits to multiple snp id, after using the highest bit score hit,
            #   The other second_snp_id would have a differnt snp id, then just use the same snp id as the primary snp
            try:
              InsertMarkerDb.add_snp(this_snp_id, int(second_hit['chrom_id']), int(second_hit['snp_position']), secondTypeID)
            except:
              logging.warn("chrom_id %s with snp_position %s was inserted in the same blast hit"%(second_hit['chrom_id'], second_hit['snp_position']))

            #raise Exception("PROG ERROR: second_snp_id in DB is expected to be equal to the found this_snp_id (2nd_hit_chr_id, 2nd_hit_chr_pos): %s %s" \
            #    %(second_hit['chrom_id'], second_hit['snp_position']))


    @staticmethod
    def __extract_gap_hit(blast_hits_with_gap, snp_type, exceptionFH):
      blast_hits_no_gap = [] 
      for hit in blast_hits_with_gap:
        try:
          chr_pos_int = int(hit['snp_position'])
        except ValueError:
          Process_output.__handle_hitting_to_gap(hit, snp_type, exceptionFH)
          continue
        blast_hits_no_gap.append(hit)
      return blast_hits_no_gap


    @staticmethod
    def __handle_hitting_to_gap(hit, snp_type, exceptionFH):
      sGapLen = int(hit['snp_position'].split('_')[3])
      marked_sseq=list(hit['sseq'])
      marked_sseq[sGapLen-1:sGapLen]='X'
      exceptionFH.write("HIT-GAP:\t%s\n"%(hit['qseq']))
      exceptionFH.write("HIT-GAP:\t%s\t%s\t%s\n"%(''.join(marked_sseq), hit['snp_position'], snp_type))


    @staticmethod
    def __handle_trivial_flanking_seq(snp_id, vcf_line, exceptionFH):
      logging.warn("snp (%s) is in a trivial flanking seq, no blast hit, ignored!" %(snp_id))
      exceptionFH.write("NO-HIT:\t%s\n" %(vcf_line))
 


    @staticmethod
    def __handle_multiple_snp_id_hit(chromName, vcf_snp_pos, this_snp_ids, snpType, vcf_line, exceptionFH):
      logging.error("Process %s: Multiple snp id blast hit for vcf_snp (chrom, pos): (%s,%s)" %(snpType, chromName, vcf_snp_pos))
      exceptionFH.write("HIT-MULTI-SNP-ID:\t%s\t%s"%(snpType, vcf_line))
      for dup_snp_id in this_snp_ids:
        logging.error("Hitting snp id: %s" %(dup_snp_id))
        exceptionFH.write("\t%s" %(dup_snp_id))
      exceptionFH.write("\n")


    @staticmethod
    def __handle_no_primary_hit(chromName, vcf_snp_pos, this_snp_ids, vcf_line, exceptionFH):
      logging.error("No primary hit found (chrom, pos): (%s,%s)" %(chromName, vcf_snp_pos))
      exceptionFH.write("NO-PRIM-HIT:\t%s\n"%(snpType, vcf_line))



    @staticmethod
    def process_primary_repeat(org_snp_hit, this_snp_id, vcf_line, cult_id_list):

      # add this snp as primary-repeat
      repeatTypeID = QueryMarkerDb.get_snp_type_id('primary-repeat')
      this_snp_loc_id = org_snp_hit['snp_loc_id']
      if this_snp_loc_id is None: 
        # when it is not a snp in db, add it as a primary-repeat
        this_snp_loc_id = InsertMarkerDb.add_snp(this_snp_id, int(org_snp_hit['chrom_id']), int(org_snp_hit['snp_position']), repeatTypeID) 
      else:
        # when it was already a snp, update it to primary-repeat
        QueryMarkerDb.update_snp_type(this_snp_loc_id, repeatTypeID)

      Process_output.upload_cultivar(this_snp_loc_id, vcf_line, cult_id_list)

      # handle the change for all primary and secondary snps with this snpID 
      primaryTypeID = QueryMarkerDb.get_snp_type_id('primary')
      secondTypeID = QueryMarkerDb.get_snp_type_id('secondary')

      # returns a list of snp_loc and each element is a list of [0]=snp_loc_id, [1]=snp_type
      db_snp_loc_list = QueryMarkerDb.get_all_snp_loc(this_snp_id)
      for db_snp_loc in db_snp_loc_list:
        db_snp_loc_id = db_snp_loc[0]
        db_snp_loc_type = db_snp_loc[1]
        if (db_snp_loc_type == primaryTypeID):
          QueryMarkerDb.update_snp_type(db_snp_loc_id, repeatTypeID)
        elif (db_snp_loc_type == secondTypeID):
          QueryMarkerDb.delete_snp_loc(db_snp_loc_id)
        elif (db_snp_loc_type == repeatTypeID):
          continue
        else:
          raise Exception("Unknown snp_type_id %d when snp_loc_id %d" %(db_snp_loc_type, db_snp_loc_id))


    @staticmethod
    def __get_unique_snp_id(this_snp_ids):
      if len(this_snp_ids) == 1:
        # when blast to only a single existing snp id 
        this_snp_id = this_snp_ids[0]
      elif len(this_snp_ids) == 0:
        # no hitting any existing snp id, get a new id for this repeat 
        new_snp_nameid = QueryMarkerDb.get_new_snp_nameid()
        this_snp_id = InsertMarkerDb.add_snp_name(config['new_snp_prefix'], new_snp_nameid, None)
      else:
        # hitting multiple snp id 
        return None
      return this_snp_id



    @staticmethod
    def __fill_snp_info(blast_hits):
      """
      Only use blash_hits with no gaps
      """
      found_snp_id = []
      repeatTypeID = QueryMarkerDb.get_snp_type_id('primary-repeat')
      isHitRepeat = False
      for hit in blast_hits:
        (this_snp_id, this_snp_loc_id, this_snp_type_id) = QueryMarkerDb.get_snp_in_chr(int(hit['chrom_id']), int(hit['snp_position']))
        hit['snp_id']=this_snp_id
        hit['snp_loc_id']=this_snp_loc_id
        hit['snp_type_id']=this_snp_type_id
        if this_snp_id is not None:
          if this_snp_id not in found_snp_id:
            found_snp_id.append(this_snp_id)
          if this_snp_type_id == repeatTypeID:
            isHitRepeat = True
      return found_snp_id, isHitRepeat

               
    @staticmethod
    def upload_cultivar(snp_loc_id, vcf_line, cult_id_list):
      vcf_list = vcf_line.split("\t")
      allele_handler = Vcf_allele_handler(vcf_list[3], vcf_list[4], vcf_list[8])
      vcf_cult_list = filter(None, vcf_list[9:])
            
      cultInfoList=[]
      for index, vcf_cult in enumerate(vcf_cult_list):
        allele = allele_handler.get_haploid_allele(vcf_cult)
        id_cult = cult_id_list[index]
                
        if allele and not QueryMarkerDb.is_cultivar_in_db(snp_loc_id, id_cult):
          cultInfoList.append([id_cult, allele, snp_loc_id])

      if (cultInfoList):
        InsertMarkerDb.add_cultivar_batch(cultInfoList)

                

if __name__ == "__main__":
    #blast_hits = Process_output.parse_blast("/home/appbioadm/testSets/testSecondaryCriterion/out-table.blast")
    #blast_hits = Process_output.parse_blast("/ws/appShare/UniSNP/data/tmp/markerdb_4Yw0DSpLfDf9nmnUqjgA/out.blast")
    InsertMarkerDb.init_db()  # for testing
    fileDir="/home/uqccha18/unisnpdb_data/tmp/20140807_snp_I3mVRkemsw"
    blast_hits = Process_output.parse_blast(os.path.join(fileDir, "out.blast"))
    Process_output.parse_vcf_after_blast(1, 1, blast_hits, os.path.join(fileDir, 'new_snp_file.vcf'))
    #Process_output.handle_aligned_to_gap('ACGTACGT', 100, 'A---ACGT', 3)


