#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.1"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"

from mysqldbPool import mysqldbPool
from config import config
from time import localtime, strftime
from collections import defaultdict


class QueryMarkerDb:

    _sql = mysqldbPool()
    __snp_type_id={}
    __snp_cache={}   # key is a tuple (chrom_id, snp_pos) and value is a tuple (snp_id, snp_loc_id, snp_type_id)
    __cultivar_cache={}   # key is a tuple (snp_loc_id, cult_name_id) and value is [cults_id, Allele]
    __cached_chr_id=None


    @staticmethod
    def init_cultivar_cache(chrID=None):
      if chrID is None:
        cultivars = QueryMarkerDb._sql.statement("""SELECT `T_SNP_Loc_ID`, `T_Cult_Name_ID_Cult`
                                                    FROM `T_Cults`""")
        QueryMarkerDb.__cached_chr_id = -1
      else:
        cultivars = QueryMarkerDb._sql.statement("""SELECT `T_SNP_Loc_ID`, `T_Cult_Name_ID_Cult`
                                                    FROM `T_SNP`, `T_Cults`
                                                    WHERE `T_Chromosome_Chromosome_ID` = %s
                                                    AND `Loc_ID` = `T_SNP_Loc_ID`; """, (chrID))
        QueryMarkerDb.__cached_chr_id = chrID

      QueryMarkerDb.__cultivar_cache={}
      for cultivar in cultivars:
        QueryMarkerDb.__cultivar_cache[(cultivar[0],cultivar[1])]=True


    @staticmethod
    def get_chr_id_cached_cult():
      return QueryMarkerDb.__cached_chr_id


    @staticmethod
    def add_cultivar_to_cache(snp_loc_id, cult_name_id):
      if (snp_loc_id, cult_name_id) not in QueryMarkerDb.__cultivar_cache:
        QueryMarkerDb.__cultivar_cache[(snp_loc_id, cult_name_id)]=True
      return


    @staticmethod
    def init_snp_cache():
      snps = QueryMarkerDb._sql.statement(""" SELECT `T_Chromosome_Chromosome_ID`, `Chrom_Pos`, 
                                                     `T_SNP_Name_ID_SNP`, `Loc_ID`, `T_SNP_Type_ID_Type`
                                              FROM `T_SNP`""")
      QueryMarkerDb.__snp_cache={}
      for snp in snps:
        QueryMarkerDb.__snp_cache[(snp[0],snp[1])] = (snp[2], snp[3], snp[4]) 


    @staticmethod
    def add_snp_to_cache(chrom_id, snp_pos, snp_id, snp_loc_id, snp_type_id):
      if (chrom_id, snp_pos) not in QueryMarkerDb.__snp_cache:
        QueryMarkerDb.__snp_cache[(chrom_id, snp_pos)]= (snp_id, snp_loc_id, snp_type_id)
        return
      elif (snp_id, snp_loc_id, snp_type_id) == QueryMarkerDb.__snp_cache[(chrom_id, snp_pos)]:
        return
      else:
        raise Exception("Conflict in inserting snp (%d %d %d %d %d) to cache with snp_id %s" \
            %(chrom_id, snp_pos, snp_id, snp_loc_id, snp_type_id, str(QueryMarkerDb.__snp_cache[(chrom_id, snp_pos)])))
        

    @staticmethod
    def get_snp_in_chr(chr_id, chr_pos):
      """
      return a tuple of (snp_id, snp_loc_id, snp_type_id)
      """
      if (chr_id, chr_pos) in QueryMarkerDb.__snp_cache:
        return QueryMarkerDb.__snp_cache[(chr_id, chr_pos)]
      else:
        return None, None, None

    @staticmethod
    def is_snp_in_db(chrom_id, snp_pos):
      """
      return None or the snp_id at this position
      """
      return QueryMarkerDb.get_snp_in_db(chrom_id, snp_pos)[0]



    @staticmethod
    def is_cultivar_in_db(snp_loc_id, cult_id):
      return (snp_loc_id, cult_id) in QueryMarkerDb.__cultivar_cache


    @staticmethod
    def init_snp_type_id_cache():
      snp_types = QueryMarkerDb._sql.statement("""SELECT `Name`, `ID_Type`
                                                     FROM `T_SNP_Type`""")
      QueryMarkerDb.__snp_type_id={}
      for snp_type in snp_types:
        QueryMarkerDb.__snp_type_id[snp_type[0].lower()]=snp_type[1]


    @staticmethod
    def get_snp_type_id(snp_type):
      if snp_type.lower() in QueryMarkerDb.__snp_type_id:
        return QueryMarkerDb.__snp_type_id[snp_type.lower()]
      return None


    @staticmethod
    def get_snp_type_id_from_db(snp_type):
      type_id = QueryMarkerDb._sql.statement("""SELECT `ID_Type`
                                      FROM `T_SNP_Type`
                                      WHERE `Name`=%s""", (snp_type))
      if (len(type_id) == 0):
        return None
      return type_id[0][0]


    @staticmethod
    def get_species():
        """
        @return list of all species, every list item contains a tuple with every column from `T_Species_Name`
        """
        return QueryMarkerDb._sql.statement("""SELECT `ID_Species`, `Species_Name`, `Species_Abbreviation`
                                                FROM `T_Species_Name`
                                             """)

    @staticmethod
    def get_species_copy_number(sp_id):
        copy_num = QueryMarkerDb._sql.statement("""SELECT Copy_Number
                                                FROM `T_Species_Name`
                                                WHERE `ID_Species` = %s
                                             """, (sp_id))
        if (len(copy_num) == 0):
          return None
        return copy_num[0][0]

    @staticmethod
    def get_number_of_ref(sp_id):
        return QueryMarkerDb._sql.statement("""SELECT COUNT(Reference_ID)
                                                FROM `T_Reference`
                                                WHERE `T_Species_Name_ID_Species` = %s
                                             """, (sp_id))[0][0]

    @staticmethod
    def get_species_name(sp_id):
        species_name = QueryMarkerDb._sql.statement("""SELECT Species_Name
                                                FROM `T_Species_Name`
                                                WHERE `ID_Species` = %s
                                             """, (sp_id))
        if (len(species_name) == 0):
          return None
        return species_name[0][0]


    @staticmethod
    def get_ref_name(ref_id):
        ref_name = QueryMarkerDb._sql.statement("""SELECT Reference_Name
                                                FROM `T_Reference`
                                                WHERE `Reference_ID` = %s
                                             """, (ref_id))
        if (len(ref_name) == 0):
          return None
        return ref_name[0][0]

    @staticmethod
    def get_chrom_name(chrom_id):
        chrom_name = QueryMarkerDb._sql.statement("""SELECT Chromosome_Name
                                                       FROM T_Chromosome
                                                       WHERE Chromosome_ID = %s
                                                   """, (chrom_id))
        if (len(chrom_name) == 0):
          return None
        return chrom_name[0][0]


    @staticmethod
    def get_new_snp_nameid():
      new_id = QueryMarkerDb._sql.statement("""SELECT MAX(`SNP_Name`)+1 FROM `T_SNP_Name` """)[0][0]
      # initialising the id
      if new_id is None:
        return 1
      return new_id


    @staticmethod
    def get_species_for_ref_id(ref_id):
        """
        Get all species for a given reference id
        @param ref_id (int): `T_Refernece`.`Reference_ID`
        @return tuple with all items from table `T_Species_Name` for the given ref_id
        """
        return QueryMarkerDb._sql.statement("""
            SELECT `ID_Species`, `Species_Name`, `Species_Abbreviation`
            FROM `T_Species_Name`
            WHERE `ID_Species` = (SELECT `T_Species_Name_ID_Species`
                                 FROM `T_Reference`
                                 WHERE `Reference_ID` = %s)
            """, (ref_id))[0]
                                        

    @staticmethod
    def _get_species_id_for_name(name):
        return QueryMarkerDb._sql.statement("""SELECT `ID_Species`
                                                FROM `T_Species_Name`
                                                WHERE `Species_Name` = %s
                                                LIMIT 1;
                                             """, (name))
    

    @staticmethod
    def _get_snp_id_for_name(prefix, name):
        return QueryMarkerDb._sql.statement("""SELECT `ID_SNP`
                                              FROM `T_SNP_Name`
                                              WHERE `PREFIX` = %s
                                                AND `SNP_Name` = %s
                                                LIMIT 1;
                                           """, (prefix, name))
    

    @staticmethod
    def _get_ref_id_for_ref_name(name):
        return QueryMarkerDb._sql.statement("""SELECT `Reference_ID`
                                              FROM `T_Reference`
                                              WHERE `Reference_Name` = %s
                                              LIMIT 1;
                                              """, (name))

    @staticmethod
    def get_references_simple():
        return QueryMarkerDb._sql.statement("""SELECT `Reference_ID`, `Reference_Name`, `Species_Name`
                                                FROM `T_Reference`
                                                JOIN `T_Species_Name`
                                                ON `T_Species_Name`.`ID_Species` = `T_Reference`.`T_Species_Name_ID_Species`;""")


    @staticmethod
    def get_snp_name(snp_id):
      snp_pad_name=QueryMarkerDb.__snp_pad_name()
      snp_name = QueryMarkerDb._sql.statement("""SELECT %s
                                              FROM `T_SNP_Name`
                                              WHERE `T_SNP_Name`.`ID_SNP` = %s""" %(snp_pad_name, snp_id))
      if (len(snp_name)==0):
        return None
      return snp_name[0][0]



    @staticmethod
    def is_repeat_snp(snp_id):
      repeatTypeID = QueryMarkerDb.get_snp_type_id('primary-repeat')
      result = QueryMarkerDb._sql.statement("""SELECT `Loc_ID` FROM `T_SNP`
                                        WHERE `T_SNP_Name_ID_SNP`= %s
                                        AND T_SNP_Type_ID_Type= %s """ %(snp_id, repeatTypeID))
      if (len(result) > 0):
        return True
      return False



    @staticmethod
    def get_upload_type_id(upload_type):
      upload_type_id = QueryMarkerDb._sql.statement("""SELECT `Upload_Type_ID`
                                             FROM `T_Upload_Type`
                                             WHERE `Upload_Type_Name` = "%s" """ %(upload_type))
      if (len(upload_type_id) == 0):
        return None
      return upload_type_id[0][0]


    @staticmethod
    def get_job_status_id(job_status):
      job_status_id = QueryMarkerDb._sql.statement("""SELECT `Job_Status_ID`
                                             FROM `T_Job_Status`
                                             WHERE `Job_Status_Name` = "%s" """ %(job_status))
      if (len(job_status_id) == 0):
        return None
      return job_status_id[0][0]


    @staticmethod
    def get_start_job():
      startJobStatusID = QueryMarkerDb.get_job_status_id("run")
      sqlStatement = """SELECT `Job_ID`, `Species_ID`, `Ref_ID`, `Upload_Type_ID`, `Upload_Subdir`
                        FROM `T_Upload_Jobs`
                        WHERE `Job_Status_ID`=%s 
                        ORDER BY `Job_ID`""" 
      values=(startJobStatusID)
      startJobs = QueryMarkerDb._sql.statement(sqlStatement, values)
      return startJobs


    @staticmethod
    def get_next_new_job():
      newJobStatusID = QueryMarkerDb.get_job_status_id("queue")
      sqlStatement = """SELECT `Job_ID`, `Species_ID`, `Ref_ID`, `Upload_Type_ID`, `Upload_Subdir`
                        FROM `T_Upload_Jobs`
                        WHERE `Job_Status_ID`=%s 
                        ORDER BY `Job_ID`""" 
      values=(newJobStatusID)
      newJobs = QueryMarkerDb._sql.statement(sqlStatement, values)
      if (len(newJobs) == 0):
        return None
      return newJobs[0]


    @staticmethod
    def get_job(jobID):
      sqlStatement = """SELECT `Job_ID`, `Species_ID`, `Ref_ID`, `Upload_Type_ID`, `Upload_Subdir`
                        FROM `T_Upload_Jobs`
                        WHERE `Job_ID`=%s"""
      values=(jobID)
      job = QueryMarkerDb._sql.statement(sqlStatement, values)
      if (len(job) == 0):
        return None
      return job[0]


    @staticmethod
    def get_references_expanded():
        return QueryMarkerDb._sql.statement("""SELECT `T_Species_Name`.`Species_Name`, `T_Species_Name`.`Species_abbreviation`,`T_Reference`.`Reference_Name`, `T_Chromosome`.`Chromosome_Name`, `T_Chromosome`.`Chromosome_ID`
                                                FROM `T_Reference`
                                                JOIN `T_Species_Name`
                                                ON `T_Species_Name`.`ID_Species` = `T_Reference`.`T_Species_name_ID_Species`
                                                JOIN `T_Chromosome`
                                                ON `T_Chromosome`.`T_Reference_Reference_ID` = `T_Reference`.`Reference_ID`
                                             """)


    @staticmethod
    def __snp_pad_name():
      return """CONCAT( `T_SNP_Name`.`PREFIX`, LPAD( `T_SNP_Name`.`SNP_Name`, %s, '0')) AS `SNP_Name`""" %(config['snp_id_length'])


    @staticmethod
    def __get_snps_query():
        return """SELECT `T_SNP`.`Loc_ID`, %s,
                         `T_SNP_Type`.`Name`, `T_Reference`.`Reference_Name`, `T_Chromosome`.`Chromosome_Name`, `T_SNP`.`Chrom_Pos`
                  FROM `T_SNP_Name`
                  JOIN `T_SNP`
                  ON `T_SNP`.`T_SNP_Name_ID_SNP` = `T_SNP_Name`.`ID_SNP`
                  JOIN `T_Chromosome`
                  ON `T_Chromosome`.`Chromosome_ID` = `T_SNP`.`T_Chromosome_Chromosome_ID` 
                  JOIN `T_Reference`
                  ON `T_Reference`.`Reference_ID` = `T_Chromosome`.`T_Reference_Reference_ID` 
                  JOIN `T_SNP_Type`
                  ON `T_SNP`.`T_SNP_Type_ID_Type` = `T_SNP_Type`.`ID_Type` """ %(QueryMarkerDb.__snp_pad_name())

    @staticmethod
    def get_snps_between_positions(start, stop, chrom_id):
        statement = QueryMarkerDb.__get_snps_query() + """WHERE `T_SNP`.`Chrom_Pos` BETWEEN %s AND %s
                                                          AND `T_Chromosome`.Chromosome_ID = %s """
        values = (int(start), int(stop), int(chrom_id))
        return QueryMarkerDb._sql.statement(statement, values)


    @staticmethod
    def get_snps_for_name(prefix, snp_name):
        return QueryMarkerDb._sql.statement(QueryMarkerDb.__get_snps_query() + "WHERE `T_SNP_Name`.`PREFIX` = %s AND `T_SNP_Name`.`SNP_Name` = %s", (prefix, snp_name))


    @staticmethod
    def get_cultivars_for_snp_id(loc_id):
        return QueryMarkerDb._sql.statement("""SELECT `T_Cult_Name`.`Cult_Name`, `T_Cults`.`Cult_Allele`
                                                FROM `T_Cults`
                                                JOIN `T_Cult_Name`
                                                ON `T_Cults`.`T_Cult_name_ID_Cult` = `T_Cult_Name`.`ID_Cult`
                                                WHERE `T_Cults`.`T_SNP_Loc_ID` = %s
                                             """, (loc_id))

    @staticmethod
    def get_cultivar_names():
        return QueryMarkerDb._sql.statement("""SELECT `Cult_Name`
                                                FROM `T_Cult_Name`""")

    @staticmethod
    def get_cultivar_id(cult_name):
      cult_id = QueryMarkerDb._sql.statement("""SELECT `ID_Cult`
                                              FROM `T_Cult_Name`
                                              WHERE `Cult_Name` = %s""", (cult_name))
      if (len(cult_id) == 0):
        return None
      return cult_id[0][0]



    @staticmethod
    def compare_cultivars(cult1, cult2, filter_x = False):
        statement = """select Loc_ID, PREFIX, SNP_Name, Source, T_Reference.Reference_Name, Chromosome_Name, Chrom_pos, h.Cult_Name, a.Cult_Allele %s, i.Cult_Name, b.Cult_Allele %s
                       from T_Cults a
                       inner join T_Cults b on a.T_SNP_Loc_ID = b.T_SNP_Loc_ID
                       inner join T_SNP c on a.T_SNP_Loc_ID = c.Loc_ID
                       inner join T_SNP_Name d on c.T_SNP_Name_ID_SNP = d.ID_SNP
                       inner join T_Chromosome e on e.Chromosome_ID = c.T_Chromosome_Chromosome_ID
                       inner join T_Reference f on e.T_Reference_Reference_ID = f.Reference_ID
                       inner join T_Species_Name g on f.T_Species_Name_ID_Species = g.ID_Species
                       INNER JOIN `T_Cult_Name` h on h.ID_Cult = a.T_Cult_Name_ID_Cult
                       INNER JOIN `T_Cult_Name` i on i.ID_Cult = b.T_Cult_Name_ID_Cult
                       INNER JOIN `T_Reference`
                       ON `T_Reference`.`Reference_ID` = e.`T_Reference_Reference_ID`
                       WHERE h.Cult_Name = %s
                       AND i.Cult_Name = %s """
        if filter_x:
            statement += """and a.Cult_Allele  not like '%%X%%'
                            and b.Cult_Allele  not like '%%X%%'"""
        
        values = (cult1, cult2,cult1, cult2)
        return QueryMarkerDb._sql.statement(statement, values)

    

    @staticmethod
    def get_chrom_id(chr_name, ref_id):
        """
        @param chr_name (str): `T_Chromosome`.`Chromosome_Name`
        @param ref_id (int): `T_Reference`.`Reference_ID`
        @return chromosome_id (int): `T_Chromosome`.`Chromosome_ID`
        """
        statement = """ SELECT `Chromosome_ID`
                        FROM `T_Chromosome`
                        WHERE `Chromosome_Name` = %s
                        AND `T_Reference_Reference_ID` = %s
                        LIMIT 1; """
        values = (chr_name, ref_id)
        results = QueryMarkerDb._sql.statement(statement, values)
        if results == ():
            return None
        else:
            return results[0][0]



    @staticmethod
    def get_all_snp_loc(snp_id):
      return QueryMarkerDb._sql.statement("""
                  SELECT `Loc_ID`, `T_SNP_Type_ID_Type` 
                  FROM `T_SNP`
                  WHERE `T_SNP_Name_ID_SNP` = %s
                  """, (snp_id))


    @staticmethod
    def get_snp_loc_id(chrom_pos, chrom_id):
      statement = """ SELECT `Loc_Id`
                      FROM `T_SNP`
                      WHERE `Chrom_Pos` = %s
                      AND `T_Chromosome_Chromosome_ID` = %s
                      LIMIT 1 """
      values = (chrom_pos, chrom_id)
      snp_loc_id = QueryMarkerDb._sql.statement(statement, values)
      if (len(snp_loc_id) == 0):
        return None
      return snp_loc_id[0][0]


    @staticmethod
    def get_chr_id_pos(snp_loc_id):
      statement ="""SELECT `T_Chromosome_Chromosome_ID`, `Chrom_Pos`
                    FROM `T_SNP`
                    WHERE `Loc_Id` = %s """
      result = QueryMarkerDb._sql.statement(statement, snp_loc_id)
      if len(result)==0:
        return None
      return result[0]
                     

    @staticmethod
    def is_cult_name_in_db(cult_name):
        """
        Check if cultivar name in database
        @return None if snp not in db, otherwise cultivar_id
        """
        statement = """SELECT `ID_Cult` FROM `T_Cult_Name` WHERE `Cult_Name` = %s """
        values = (cult_name)
        result = QueryMarkerDb._sql.statement(statement, values)
        if len(result) == 0:
            return None
        else:
            return result[0][0]


    @staticmethod
    def update_snp_type(snp_loc_id, snp_type_id):
      """
      The function updates the snp 
      @param snp_id: the snp needs to be updated
      """
      curDateTime = strftime("%Y-%m-%d %H:%M:%S", localtime())
      return QueryMarkerDb._sql.statement(""" UPDATE `T_SNP`
                  SET `T_SNP_Type_ID_Type`=%s, `Update_Date`=%s
                  WHERE `Loc_ID` = %s """, (snp_type_id, curDateTime, snp_loc_id))


    @staticmethod
    def update_job_status(jobID, job_status_id):
      startStatusID = QueryMarkerDb.get_job_status_id("run")
      finishStatusID = QueryMarkerDb.get_job_status_id("done")
      errorStatusID = QueryMarkerDb.get_job_status_id("error")

      curDateTime = strftime("%Y-%m-%d %H:%M:%S", localtime())
      sql_statement="UPDATE `T_Upload_Jobs` SET `Job_Status_ID`=%s"
      values=[job_status_id]
      if (job_status_id == startStatusID):
        sql_statement += ", `Start_Date_Time`=%s"
        values.append(curDateTime)
      elif (job_status_id == finishStatusID or job_status_id == errorStatusID):
        sql_statement += ", `End_Date_Time`=%s"
        values.append(curDateTime)
      else:
        raise Exception("Unknow job status id: %s" %(job_status_id))

      sql_statement += " WHERE `Job_ID`=%s"
      values.append(jobID)

      return QueryMarkerDb._sql.statement( sql_statement, tuple(values) )


    @staticmethod
    def delete_cultivar(snp_loc_id):
      cult_list=[]
      for (loc_id, cult_id) in QueryMarkerDb.__cultivar_cache:
        if loc_id == snp_loc_id:
          cult_list.append(cult_id)
      # check if snp_loc_id in cultivar_cache
      if cult_list:
        # if yes, delete it from cache
        for thisCultID in cult_list:
          del QueryMarkerDb.__cultivar_cache[(snp_loc_id, thisCultID)]
        # delete it from DB
        QueryMarkerDb._sql.statement("""DELETE FROM `T_Cults` WHERE `T_SNP_Loc_ID` = %s """, (snp_loc_id))
        return True
      else:
        return None



    @staticmethod
    def delete_snp_loc(snp_loc_id):
      # Delete it from the cache
      result = QueryMarkerDb.get_chr_id_pos(snp_loc_id)
      if result is None:
        return None
      (chr_id, chr_pos) = (result[0], result[1])
      del QueryMarkerDb.__snp_cache[(chr_id, chr_pos)]
      # And from the db
      return QueryMarkerDb._sql.statement("""DELETE FROM `T_SNP` 
          WHERE `T_SNP`.`Loc_ID` = %s """, (snp_loc_id))



if __name__ == '__main__':
    # Testing
#    print "get_species_copy_number(1): %s\n" % (QueryMarkerDb.get_species_copy_number(1))
#    print "get_species_name: %s\n" % (QueryMarkerDb.get_species_name(2))
#    print "get_snp_name: %s\n" % (QueryMarkerDb.get_snp_name(4))
#    print "get_number_of_ref(2): %d\n" %(QueryMarkerDb.get_number_of_ref(2))
#    print "get_new_snp_id(): %d\n" %(QueryMarkerDb.get_new_snp_id())
#    print "get_snps_for_name('UQ', 1):"
#    print QueryMarkerDb.get_snps_for_name('UQ', 1)
#    print "\nget_snps_between_positions(91310, 91400, 40):"
#    print QueryMarkerDb.get_snps_between_positions(91310, 91400, 40)
#    QueryMarkerDb.init_snp_type_id_cache()
#    print "\nget_snp_type_id('primary2'): %s" %(QueryMarkerDb.get_snp_type_id('primary2'))
#    print "\nget_snp_type_id('secondary'): %s" %(QueryMarkerDb.get_snp_type_id('secondary'))
#
#    QueryMarkerDb.init_snp_cache()
#    print QueryMarkerDb.is_snp_in_db(40,'2196883')
#    print QueryMarkerDb.is_snp_in_db(40,'2196884')
#    print QueryMarkerDb.is_snp_in_db(40,'2196884')
#
#    print QueryMarkerDb.get_job_status_id("error")
#    print QueryMarkerDb.get_job_status_id("queue")
#    print QueryMarkerDb.get_job_status_id("run")
#    print QueryMarkerDb.get_job_status_id("error")
#    print QueryMarkerDb.get_job_status_id("done")

#    InsertMarkerDb.init_db()
    #print QueryMarkerDb.is_cult_name_in_db('SOSR02')
    #print QueryMarkerDb.is_cult_name_in_db('SOSR107')
    #QueryMarkerDb.update_snp_type(3, 1)

#    print type(QueryMarkerDb.get_ref_name(1))
#    print type(QueryMarkerDb.get_number_of_ref(1))
#    print type(True)
#    print type(1)

    #QueryMarkerDb.init_snp_cache()
    #QueryMarkerDb.init_snp_cache(1)

    QueryMarkerDb.init_cultivar_cache(1)
    QueryMarkerDb.delete_cultivar(7)
#    a = QueryMarkerDb.get_snp_in_chr(1,1234)
#    if a[0] is not None:
#      print a
#    else:
#      print "none\n"
#
#    QueryMarkerDb.delete_snp_loc(2)
#    result = QueryMarkerDb.get_snp_in_chr(1,1234) 
#    print result
#
#    a = QueryMarkerDb.get_snp_in_chr(1,1234)
#    if a[0] is not None:
#      print a
#    else:
#      print "none\n"

    #print QueryMarkerDb.get_cultivar_id('SOSR01')
    # print QueryMarkerDb.secondary_to_primary(4)
    #print QueryMarkerDb.is_cultivar_in_db(10, 3)

    # chrom_id = 1 chrom_name = XA01_v3.0 ref_id = 1 snp_pos = 301
    #for i in range(0, 1000):
    #    QueryMarkerDb.is_cult_name_in_db('Cult_CULT1')
    #    QueryMarkerDb.get_cultivars_for_snp_id(1)
    #print QueryMarkerDb.get_chrom_id('XA01_v3.0', 1)

##    for row in QueryMarkerDb.get_references_expanded():
##        print row
##    print QueryMarkerDb.sql.statement("SELECT * FROM `T_Reference` WHERE `Reference_Name` = %s", ("testref"))
