#!/usr/bin/python

__author__ = "Kenneth Chan"
__version__ = "1.0" 
__contact__ = "<c.chan10@uq.edu.au>"


import glob
import os

from config import config 

from process_input import Process_input
from process_output import Process_output
from process_fasta_snp import Process_fasta_snp

from insertMarkerDb import InsertMarkerDb
from queryMarkerDb import QueryMarkerDb

import traceback

import logging
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__),'../log','unisnpdb.log'), level=config['logLevel'], format="%(asctime)s:%(levelname)s:%(message)s")


class Upload_manager:

  def __init__(self):
    pass

  @staticmethod
  def add_new_upload_job(sp_id, ref_id, uploadDir, uploadType):
    uploadTypeID = QueryMarkerDb.get_upload_type_id(uploadType)
    newJobStatusID = QueryMarkerDb.get_job_status_id("queue")
    InsertMarkerDb.insert_job(newJobStatusID, sp_id, ref_id, uploadDir, uploadTypeID)


  @staticmethod
  def process_jobs():
    startedJobs = QueryMarkerDb.get_start_job()
    # There is a process_jobs thread already, no need to start another one
    if len(startedJobs)>0:
      logging.debug("There is already a job procssing thread running, no need another one")
      return

    while 1:
      # get the next new job
      thisNewJob = QueryMarkerDb.get_next_new_job()
      # When there is no more new job
      if thisNewJob is None:
        logging.info("Finished all jobs")
        return

      # while newJob is not empty:
      jobID = thisNewJob[0]
      spID = thisNewJob[1]
      refID = thisNewJob[2]
      uploadTypeID = thisNewJob[3]
      uploadSubdir=thisNewJob[4]
      logging.info("Start processing job (jobID %d, spID %d, refID %d, uploadTypeID %d, uploadSubdir %s)"%(jobID, spID, refID, uploadTypeID, uploadSubdir)) 
      thisUploadDir = os.path.join(config['workhorse_upload_dir'], uploadSubdir)

      QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("run"))
      if uploadTypeID == QueryMarkerDb.get_upload_type_id("vcf"):
        try:
          logging.info("Obtain *_org.vcf from %s" %(thisUploadDir))
          vcfFiles = glob.glob( os.path.join(thisUploadDir, "*_org.vcf"))
          # decide cultivar info on which chromosome to be cached
          thisChrName = Upload_manager.peepChrNameinVcf(vcfFiles[0])
          thisChrID = QueryMarkerDb.get_chrom_id(thisChrName, refID)
          cachedChrID = QueryMarkerDb.get_chr_id_cached_cult()

          # cachedChrID is -1 when all chromosomes have been cached
          if cachedChrID is None or cachedChrID != -1 or cachedChrID != thisChrID:
            logging.info("Caching cultivar info for chr_id %d" %(thisChrID))
            cachedChrID = thisChrID
            QueryMarkerDb.init_cultivar_cache(thisChrID)
            logging.info("Finished caching cultivar info for chr_id %d" %(thisChrID))

          logging.info("Process_input.extract_new_snp_in_vcf(%s, %s)"%(refID, vcfFiles[0]))
          new_vcf_fn = Process_input.extract_new_snp_in_vcf(refID, vcfFiles[0])

          logging.info("Upload_manager.__process_vcf_file(%s, %s, %s, %s)" %(spID, refID, thisUploadDir, new_vcf_fn))
          Upload_manager.__process_vcf_file(spID, refID, thisUploadDir, new_vcf_fn)


          ##### new job processing finished here

          QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("done"))

        except Exception as e:
          logging.error("Upload VCF exception value: %s" %(str(e)))
          logging.error(traceback.format_exc())
          QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("error"))
      elif uploadTypeID == QueryMarkerDb.get_upload_type_id("ref"):
        try:
          refFastaFile = os.path.join(thisUploadDir, "ref_genome.fasta")
          offsetFile = os.path.join(thisUploadDir, "ref_offset.gff3")
          logging.info("Upload_manager.__process_ref_file(%s, %s, %s, %s)" %(spID, refID, refFastaFile, offsetFile))
          Upload_manager.__process_ref_file(spID, refID, refFastaFile, offsetFile) 
          QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("done"))
        except Exception as e:
          logging.error("Upload ref exception value: %s" %(str(e)))
          logging.error(traceback.format_exc())
          QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("error"))
      else:
        raise Exception("Unknown upload type: %s" %(uploadTypeID))
        logging.error(traceback.format_exc())
        QueryMarkerDb.update_job_status(jobID, QueryMarkerDb.get_job_status_id("error"))

      logging.info("Finished processing job (jobID %d, spID %d, refID %d, uploadTypeID %d, uploadSubdir %s)"%(jobID, spID, refID, uploadTypeID, uploadSubdir)) 



  @staticmethod
  def peepChrNameinVcf(vcfFile):
    vcf_fh = open(vcfFile, 'r')

    for line in vcf_fh:
      line = line.replace("\n", "")
      if not line.startswith('#'):
        tabbed = line.split("\t")
        chrom_name = tabbed[0]
        vcf_fh.close()
        return chrom_name


  @staticmethod
  def __process_ref_file(species_id, ref_id, ref_fn, offset_fn):
    input_processor = Process_input(ref_id, species_id)
    input_processor.add_ref_to_ref_file(ref_fn, offset_fn)
    input_processor.index_db_ref_file()


  @staticmethod
  def __process_vcf_file(species_id, reference_id, upload_dir, vcf_fn):
    if config['workhorse_user'] is None:
      db_file = os.path.join(config['data_dir'], str(species_id), config['ref_db_fn'])
    else:
      db_file = os.path.join(config['workhorse_data_dir'], str(species_id), config['ref_db_fn'])


    ##### run-stage1
    fasta_processor = Process_fasta_snp(vcf_fn)
    flanking_output_fn = os.path.join(upload_dir, "flanking_regions.fasta")
    logging.info("get_flanking_regions(%s, %s, %s)"% (flanking_output_fn, db_file, reference_id))
    fasta_processor.get_flanking_regions(flanking_output_fn, db_file, reference_id)

    ### submit a mapper-spawn job for this upload_dir
    ###  if qsub sucessful, update job stage to check_new_snp_done
    ###  if qsub returns error, update job stage to check_new_snp_error, continue




    ##### This part make it parallel
    ##### run-stage2
    ##### CHANGE
    out_blast = os.path.join(upload_dir, 'out.blast')
    input_processor = Process_input(reference_id, species_id)
    logging.info("run_mapper(%s, %s)"% (flanking_output_fn, out_blast))
    input_processor.run_mapper(flanking_output_fn, out_blast)

    ### Need to handle part of the decision making in parse_vcf_after_blast()

    ##### END



    ##### run-stage3 process

    logging.info("Process_output.parse_blast(%s)" %(out_blast)) 
    blast_hits = Process_output.parse_blast(out_blast)

    logging.info("Process_output.parse_vcf_after_blast(%d, %d, blast_hits, %s)" %(species_id, reference_id, vcf_fn))
    Process_output.parse_vcf_after_blast(species_id, reference_id, blast_hits, vcf_fn)
    del blast_hits    # no longer need it, force garbage collection
    logging.info("Finished Process_output.parse_vcf_after_blast")

if __name__ == '__main__':
  logging.info("Initialising snp type id cache")
  QueryMarkerDb.init_snp_type_id_cache()
  logging.info("Initialising snp cache")
  QueryMarkerDb.init_snp_cache()
  logging.info("Start processing jobs")
  Upload_manager.process_jobs()
