#!/usr/bin/python

__author__ = "Lars Smits, Kenneth Chan"
__version__ = "1.0"
__contact__ = "<smits_lars@outlook.com>, <c.chan10@uq.edu.au>"

class Vcf_allele_handler:

    def __init__(self, ref, alt, vcf_format):
        """
        This script was designed for VCF version 4.0, but it might also work for other versions.

        example vcf line:
        chrC09	562	chrC090000001	T	G	2	.	DP=128;PL=chrC09	GT:DP	.|.:0	.|.:0	.|.:0

        @param ref: Reference allele, in the example above ref would be: T
        @param alt: ALT column in VCF line. In the given example above alt would be: G
        @param vcf_format: FORMAT column in VCF line. In the given example, vcf_format would be: GT:DP
        @return None
        """
        self.__refs = tuple(ref.replace(",", ""))
        self.__alts = tuple(alt.replace(",", ""))
        self.__vcf_format_list = vcf_format.split(":")
        

    def get_feature_out_format(self, field, vcf_cult):
        """
        Get only the specified output from the cultivar column
        If the format column specifies GT:DP and we give GT as field and .|.:0 as vcf_cult
        this method will only return .|.
        If DP would have been selected as field, the output would be 0
        @param field: The field you want to get out of the cultivar column
                      Note: this item must be in the FORMAT column, case sensitive!
        @param vcf_cult: vcf cultivar column e.g. .|.:0
        @return feature_out: The cultivar column filtered for only the output specefied
        """
        vcf_cult_list = vcf_cult.split(":")
        if field in self.__vcf_format_list:
            return vcf_cult_list[self.__vcf_format_list.index(field)]
        else:
            return None

    def __calculate_allele(self, allele):
        """
        You will need to execute this method twice.
        For example if the genotype information in the VCF is 0|1
        you will need to execute this method for both 0 and 1
        @param allele (str): see description above, e.g. 1 don't
                             give the allele as a string
        @return allele (str): in human readable format e.g. A
        """
        if allele == '0':
            allele = self.__refs[0]
        elif allele == '.':
            allele = 'X'
        else:
            allele = int(allele)
            allele = self.__alts[allele-1]
        return allele
    
    def __format_gt(self, gt):
        """
        Get genotype information
        @param gt: vcf genotype information e.g. 0|1 or .|. or 1|1, etc.
        @return tuple: (allele1 (str), allele2 (str), phased (bool))
        """
        allele1 = self.__calculate_allele(gt[0])
        phased = gt[1] == "|"
        allele2 = self.__calculate_allele(gt[2])
        return (allele1, allele2, phased)
        
    
    def get_alleles(self, vcf_cult):
        """
        @param vcf_cult: cultivar in vcf format, for example 0|1:7 or .|.:0
                         Note: a vcf line often has more than one vcf cultivar.
                               Usually the last columns in the VCF line
        @return formatted: Formatted output in human readable format
                           e.g. G/A
        """
        gt = self.get_feature_out_format('GT', vcf_cult)
        if gt:
            formatted = "/".join(self.__format_gt(gt)[0:-1])
            return formatted
        else:
            return None
        

    def get_haploid_allele(self, vcf_cult):
        """
        @param vcf_cult: cultivar in vcf format, for example 0|0:7 or .|.:0
                         Note: a vcf line often has more than one vcf cultivar.
                               Usually the last columns in the VCF line
        @return formatted: Formatted output in human readable format
                           e.g. G
        """
        gt = self.get_feature_out_format('GT', vcf_cult)
        if gt:
            return self.__format_gt(gt)[0]
        else:
            return None
        
        

if __name__ == "__main__":
    vcf = Vcf_allele_handler('G', 'A', "GT:DP")
    print vcf.get_alleles('0|1:7')
    print vcf.get_haploid_allele('1|1:7')
    
