#!/usr/bin/python

__author__ = "Kenneth Chan, Lars Smits"
__version__ = "1.3"
__contact__ = "<c.chan10@uq.edu.au>, <smits_lars@outlook.com>"


from flask import Flask, render_template, request, redirect, url_for
from functools import wraps
import sys
import os
import random
from threading import Thread
from time import localtime, strftime



try:
    custlib = os.path.join(os.path.dirname(__file__),'./lib')
except NameError:
    custlib = 'lib'
sys.path.append(custlib)


from paramiko import SSHClient
from scp import SCPClient


# custom modules
from config import config
from queryMarkerDb import QueryMarkerDb
from insertMarkerDb import InsertMarkerDb

from upload_manager import Upload_manager

import logging
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__),'log','unisnpdb.log'), level=config['logLevel'], format="%(asctime)s:%(levelname)s:%(message)s")


app = Flask(__name__)

def get_random_str(length):
    random_str = '1234567890qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
    tmp_id = ''.join([random_str[random.randint(0, len(random_str)-1)] for x in range(0, length)])
    return tmp_id


def templated(template=None, main_template='bootstrap.tmpl'):
    def decorator(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            template_name = template
            if template_name is None:
                template_name = request.endpoint \
                    .replace('.', '/') + '.tmpl'
            ctx = f(*args, **kwargs)
            if str(type(ctx)) == "<class 'werkzeug.wrappers.Response'>":
                return ctx
            if not isinstance(ctx, dict):
                ctx = str(ctx)
            if template:
                if not isinstance(ctx, dict):
                    ctx = dict(content=ctx)
                body = render_template(template_name, **ctx)
            else:
                body = str(ctx)
            if main_template:
                return render_template(main_template, body=body)
            else:
                return body
        return decorated_function
    return decorator


@app.route("/search/")
@templated('search_page.tmpl')
def search_page():
    references = QueryMarkerDb.get_references_expanded()
    cultivars = QueryMarkerDb.get_cultivar_names()
    return dict(references=references, cultivars=cultivars)


@app.route("/search_results/", methods=["GET"])
@templated('search_results.tmpl')
def search_results():
    results = []
    add_cultivars = False
    if request.args.get('snpsearchsubmit'):
        results = list(QueryMarkerDb.get_snps_for_name(request.args.get('snpsearch')))
        add_cultivars = True
    elif request.args.get('betweensubmit'):
        start    = request.args.get('startpos')
        stop     = request.args.get('endpos')
        chrom_id = request.args.get('chrom_id')
        results  = list(QueryMarkerDb.get_snps_between_positions(start, stop, chrom_id))
        add_cultivars = True
        
    elif request.args.get('cultCompareSubmit'):
        cult1 = request.args.get('cultname1')
        cult2 = request.args.get('cultname2')
        filter_x = request.args.get('xfilter')
        results = QueryMarkerDb.compare_cultivars(cult1, cult2, filter_x)
        
    if add_cultivars:
        results = map(list, results)
        for index, row in enumerate(results):
            cultivars = [":".join(cult) for cult in QueryMarkerDb.get_cultivars_for_snp_id(row[0])]
            results[index] += cultivars
    
    return dict(results=results)


@app.route("/upload_snp/")
@templated("upload_snp.tmpl")
def upload_snp():
    species = QueryMarkerDb.get_species()
    references = QueryMarkerDb.get_references_simple()
    return dict(references_species=references, species=species)


#####################################################################
#--------------------/process_uploads functions/--------------------#
#####################################################################
def __save_upload_ref(request, ref_id, species_id):
    curDateStr=__getCurDateStr() 
    thisDirname = "%s_ref_%s" %(curDateStr,get_random_str(10))
    upload_dir = os.path.join(config['upload_dir'],thisDirname)
    os.makedirs(upload_dir)

    ref_file = request.files['reffile']
    ref_fn = os.path.join(upload_dir, 'ref_genome.fasta')
    ref_file.save(ref_fn)
    offset_file = request.files['offsetfile']
    offset_fn = os.path.join(upload_dir, 'ref_offset.gff3')
    offset_file.save(offset_fn)

    Upload_manager.add_new_upload_job(species_id, ref_id, thisDirname, "ref")

    if config['workhorse_user'] is not None:
      logging.info("workhorse_user is set, uploading to workhorse")
      # copy the local dir to workhorse dir
      thisSsh = __getSshClient(config['workhorse_servers'], config['workhorse_user'], config['workhorse_upload_dir'])
      if thisSsh is not None:
        __scp_dir_to_workhorse(thisSsh, thisDirname)
        thisSsh.close()
      else:
        logging.warn("Couldn't get ssh client, data not uploaded to workhorse")
      logging.info("Please run Upload_manager.process_jobs to actually process the submitted ref")
    else:
      logging.info("workhorse_user is not set, use local server Thread to process jobs")

      logging.info("Initialising snp type id cache")
      QueryMarkerDb.init_snp_type_id_cache()
      logging.info("Initialising snp cache")
      QueryMarkerDb.init_snp_cache()
      logging.info("Start processing jobs")
      t = Thread(target=Upload_manager.process_jobs)
      t.start()


def __getSshClient(remoteServers, user, archiveLoc):
  client = SSHClient()
  client.load_system_host_keys()
  for server in remoteServers:
    try:
      logging.debug('try '+server)
      client.connect(server, username=user)
      logging.debug('connected to '+server)
      stdin, stdout, stderr=client.exec_command('mkdir -p '+archiveLoc, timeout=60)
      if stdout.channel.recv_exit_status() != 0:
        logging.debug("Couldn't make directory, try next server: "+stderr.channel.recv_stderr(30))
        continue
      return client
    except Exception, e:
      logging.debug('Got exception, try next server: '+str(e))
      continue
  else:
    logging.info("Possibly network problem, no available server to access repository")
    return None



def __scp_dir_to_workhorse(thisSsh, sourceFile):
  local_file = os.path.join(config['upload_dir'],sourceFile)
  workhorse_dir = config['workhorse_upload_dir']

  scpClient = SCPClient(thisSsh.get_transport(), buff_size=65536, socket_timeout=120)
  scpClient.put(local_file, recursive=True, remote_path=workhorse_dir)

  


def __getCurDateStr():
  return strftime("%Y%m%d", localtime())



def __save_snp_file(request, species_id, reference_id):
    vcf_files = request.files.getlist('vcffiles[]')
    curDateStr=__getCurDateStr() 

    
    thisSsh = None
    if config['workhorse_user'] is not None:
      logging.warn("workhorse_user is not set, not uploading to workhorse.")
      thisSsh = __getSshClient(config['workhorse_servers'], config['workhorse_user'], config['workhorse_upload_dir'])

    for vcf_file in vcf_files:
        logging.debug("Process uploaded file: %s" %(vcf_file.filename))

        thisDirname = "%s_snp_%s" %(curDateStr,get_random_str(10))
        upload_dir = os.path.join(config['upload_dir'],thisDirname)
        os.makedirs(upload_dir)

        org_vcf_fn = os.path.join(upload_dir, __extract_filename(vcf_file.filename)+"_org.vcf")
        vcf_file.save(org_vcf_fn)

        Upload_manager.add_new_upload_job(species_id, reference_id, thisDirname, "vcf")

        if config['workhorse_user'] is not None:
          if thisSsh is not None:
            # upload this directory to the workhorse
            __scp_dir_to_workhorse(thisSsh, thisDirname)
          else:
            logging.warn("Couldn't get ssh client, data not uploaded to workhorse")

    if thisSsh is not None:
      logging.info("Please run Upload_manager.process_jobs to actually process the submitted snp files")
      thisSsh.close()
    else:
      logging.info("workhouse_user is not set, use local server Thread to process jobs")
      t = Thread(target=Upload_manager.process_jobs)
      t.start()


def __extract_filename(filename):
  filename = filename.strip()
  filename = filename.replace(" ", "_")
  return os.path.splitext(filename)[0]


def __get_species_id_from_req(request):
    species_id = request.form['speciesdropdown']
    
    if not species_id:
        species_name = request.form['speciesinput']
        species_abbreviation = request.form['speciesabb']
        species_id = int(InsertMarkerDb.add_species(species_name, species_abbreviation))
    else:
        species_id = int(species_id)
    return species_id


@app.route("/process_uploads_snp/", methods=["POST"])
@templated()
def process_uploads_snp():
    # only care about species id when ref_id is not found
    # Reference id always needs to be known
    # from there we can get the species information
    reference_id = int(request.form['refdropdown'])
        
    # get species id based on reference id
    # all stuff will be saved in a directory
    # with the species id as name
    species_id, specie_name, specie_abb = QueryMarkerDb.get_species_for_ref_id(reference_id)  
    
    __save_snp_file(request, species_id, reference_id)
    
    return redirect(url_for('upload_snp'))


#=========================#

@app.route("/upload_reference/")
@templated("upload_reference.tmpl")
def upload_reference():
    species = QueryMarkerDb.get_species()
    references = QueryMarkerDb.get_references_simple()
    
    return dict(references_species=references, species=species)



@app.route("/process_upload_reference/", methods=["POST"])
@templated()
def process_upload_reference():
    
    # only care about species id when ref_id is not found
    # Reference id always needs to be known
    # from there we can get the species information
    reference_id = request.form['refdropdown']
    if not reference_id:
        species_id = __get_species_id_from_req(request)
        ref_input_name = request.form['refinput']
        reference_id = InsertMarkerDb.add_reference(ref_input_name, species_id)
    else:
        reference_id = int(reference_id)

    # get species id based on reference id
    # all stuff will be saved in a directory
    # with the species id as name
    species_id, species_name, species_abb = QueryMarkerDb.get_species_for_ref_id(reference_id)
    
    __save_upload_ref(request, reference_id, species_id)
    return redirect(url_for('upload_reference'))
 

#===================================================================#

@app.route("/")
@templated()
def home():
    return "uniSNPdb" 


if __name__ == "__main__":
  InsertMarkerDb.init_db()
  if config['workhorse_user'] is None:
    print "Use local server to process job."
    print "Caching snp_type_id and snp.... Please wait."
    QueryMarkerDb.init_snp_type_id_cache()
    QueryMarkerDb.init_snp_cache()
    print "Finished caching snp_type_id and snp. Program is ready."
  print "Program started at %s" %(strftime("%Y-%m-%d %H:%M:%S", localtime()))
  app.run(port=5000, debug=False, host='0.0.0.0')

