#!/bin/bash

# Author: Kenneth Chan                                                                                 
# Version 0.2   Aug 2014
# Version 0.1   Dec 2013

#################### Default settings #################### 

outDir=0;

cpu=2;
mem='10G';
nodetype='any';
numOfRunHour=36;

numOfReadInSubFa=5000; 

blastProg='blastn';

## Example Stage: 0-runEverything; 1-run parsing only
start_stage=0;

thisID='blast';

fileExt='false';

outFormat='false';
numOfTopHit='false';
eValue='false';

parser='false';
wordSize='false';

#################### User settings #################### 


usage="\nUSAGE: bash $0 [-h] [-d] [-o <outDir>] [-t <#CPU>] [-m <memory>] [-r <#RunHour>] [-n <#Reads>] [-p <blastn|blastx>] [-s <0|1>] [-i <thisID>] [-x <fileExtension>] [-c <numOfTopHit>] [-e <E-value>] [-a <parserProgram>] [-w <wordSize>] -b <database> <fastaFile|fastqFile>\n";
usage="$usage    -h    This help\n";
usage="$usage    -d    Debug mode, do not submit job.\n";
usage="$usage    -o    Output directory. Default: output to the first input file directory.\n";
usage="$usage    -t    Num of CPU. Default ${cpu}\n";
usage="$usage    -m    Memory amount. Default ${mem}\n";
usage="$usage    -r    Number of running hour for each sub job. Default ${numOfRunHour}\n";
usage="$usage    -n    Number of reads in each sub file. Default ${numOfReadInSubFa}\n";
usage="$usage    -p    Type of blast. Either blastn or blastx. Default ${blastProg}\n";
usage="$usage    -s    Starting stage: 0-blast; 1-parse. Default ${start_stage}\n";
usage="$usage    -i    The id to be used for filename and jobID. Default ${thisID}\n";
usage="$usage    -x    The file extension, this string in filename will be replaced with the type of blast. Default use the input file extension\n";
usage="$usage    -f    Blast output format. Default is to use standard blast format\n";
usage="$usage    -c    Num of top hit to keep in the blast file. Default blast default\n";
usage="$usage    -e    E-value. Default use default blast e-value\n";
usage="$usage    -a    Parser file to be used. Default is not to parse the blast output\n";
usage="$usage    -w    Wordsize to be used in blast. Default is default blast setting\n";
usage="$usage    -b    The database file. This option is needed.\n";


DEBUG=0;
options=':hdo:t:m:r:n:p:s:i:x:f:c:e:a:w:b:';
while getopts $options OPTION
do
	case $OPTION in
		h) echo -e "$usage"; exit 0;;
		d) DEBUG=1;;
		o) outDir=${OPTARG};;
		t) cpu=${OPTARG};;
		m) mem=${OPTARG};;
		r) numOfRunHour=${OPTARG};;
		n) numOfReadInSubFa=${OPTARG};;
		p) blastProg=${OPTARG};;
		s) start_stage=${OPTARG};;
		i) thisID=${OPTARG};;
		x) fileExt=${OPTARG};;
		f) outFormat=${OPTARG};;
		c) numOfTopHit=${OPTARG};;
		e) eValue=${OPTARG};;
		a) parser=${OPTARG};;
		w) wordSize=${OPTARG};;
		b) db=${OPTARG};;
		\?) echo;echo "Unknown option -$OPTARG">&2; echo; exit 1;;
		:) echo;echo "Missing option argument for -$OPTARG">&2; echo; exit 1;;
	esac
done
shift $(( $OPTIND - 1 ));


if [ "${#@}" -le "0" ] || [ -z "$db" ];then
	echo -e "$usage"; exit 1;
fi

if [ "$outDir" == "0" ]; then
	outDir=`dirname $1`;
fi


PBSoutDir="${outDir}/pbs_out";
if [ ! -d $PBSoutDir ]; then
	echo "mkdir -p $PBSoutDir";
	mkdir -p $PBSoutDir;
fi


if [ "${fileExt}" == 'false' ]; then
	fileExt=`basename $1`;
	fileExt=${fileExt##*.};
fi


progDir="$( cd "$( dirname "${0}" )" && pwd )";

# Convert to fa file if it's fq file 
sym=`head -n1 $1 | cut -c1`
if [ "$sym" == "@" ]; then
	${progDir}/fastq_to_fasta_orgHeader.pl -file $1 -out ${1}.${fileExt}
	faFile="${1}.${fileExt}"
else
	faFile=${1}
fi

# split the fa into sub files
numOfLineInSubFa=$(( $numOfReadInSubFa * 2 ))

echo "split -l ${numOfLineInSubFa} -a 4 -d ${faFile} ${faFile}."
split -l ${numOfLineInSubFa} -a 4 -d ${faFile} ${faFile}.

# Build the file list for pbs array
fileList=${outDir}/fileList.`date +"%Y%m%d%H%M"`;
> ${fileList};
counter=0;
for a in `ls ${faFile}.*`; do
	echo $a >> ${fileList};
	counter=$(( $counter + 1 ));
done

runid="${thisID:0:10}";


tempDir=`basename "${0}"`;
tempDir="${tempDir/\.sh/}";
tempFile="${progDir}/${tempDir}.TEMPLATE";

outFile="${outDir}/${tempDir}-$runid.sh";


# Generate blast submission file
#echo "
#sed -e \"s/_CPU_/${cpu}/\" \
#	  -e \"s/_MEM_/${mem}/\" \
#		-e \"s/_NODETYPE_/${nodetype}/\" \
#	  -e \"s/_HR_/${numOfRunHour}/\" \
#		-e \"s/_BLAST-PROG_/${blastProg}/\" \
#		-e \"s/_START-STAGE_/${start_stage}/\" \
#		-e \"s/_RUNID_/${runid}/\"
#		-e \"s/_FILE-EXT_/${fileExt}/\" \
#		-e \"s/_OUT-FORMAT_/${outFormat}/\" \
#		-e \"s|_NUM-OF-TOP-HIT_|${numOfTopHit}|\" \
#		-e \"s/_E-VALUE_/${eValue}/\" \
#		-e \"s|_PARSER_|${parser}|\" \
#		-e \"s/_WORD-SIZE_/${wordSize}/\" \
#		-e \"s|_OUTDIR_|${outDir}|\" \
#		-e \"s|_PBSOUTDIR_|${PBSoutDir}|\" \
#		-e \"s|_FILE-LIST_|${fileList}|\" \
#		-e \"s|_DB_|${db}|\" < ${tempFile} > ${outFile}";
#echo;

sed -e "s/_CPU_/${cpu}/" \
	  -e "s/_MEM_/${mem}/" \
		-e "s/_NODETYPE_/${nodetype}/" \
	  -e "s/_HR_/${numOfRunHour}/" \
		-e "s/_BLAST-PROG_/${blastProg}/" \
		-e "s/_START-STAGE_/${start_stage}/" \
		-e "s/_RUNID_/${runid}/" \
		-e "s/_FILE-EXT_/${fileExt}/" \
		-e "s/_OUT-FORMAT_/${outFormat}/" \
		-e "s|_NUM-OF-TOP-HIT_|${numOfTopHit}|" \
		-e "s/_E-VALUE_/${eValue}/" \
		-e "s|_PARSER_|${parser}|" \
		-e "s/_WORD-SIZE_/${wordSize}/" \
		-e "s|_OUTDIR_|${outDir}|" \
		-e "s|_PBSOUTDIR_|${PBSoutDir}|" \
		-e "s|_FILE-LIST_|${fileList}|" \
		-e "s|_DB_|${db}|" < ${tempFile} > ${outFile}

if [ "${counter}" == "1" ]; then 
	cmd="qsub -J 1-2:3 ${outFile}";
else
	cmd="qsub -J 1-${counter} ${outFile}";
fi
echo "$cmd"; echo;
if [ ! -z $DEBUG ] && [ "$DEBUG" -lt 1 ]; then
	pbsID=`$cmd`; pbsID=`echo ${pbsID}`; pbsID=${pbsID%% *};
else
	echo "DEBUG mode, skip execution: $cmd"
fi
	


# Combine all blast results to 
combTempFile="${progDir}/${tempDir}_combine.TEMPLATE";
outFile="${outDir}/${tempDir}-${runid}_combine.sh";

inFilename=`basename $1`;
inFilename=${inFilename%.*};

sed	-e "s/_RUNID_/${runid}/" \
		-e "s|_PBSOUTDIR_|${PBSoutDir}|" \
		-e "s|_FILE-LIST_|${fileList}|" \
		-e "s|_OUTDIR_|${outDir}|" \
		-e "s/_FILE-EXT_/${fileExt}/" \
		-e "s|_PARSER_|${parser}|" \
		-e "s|_PROG-DIR_|${progDir}|" \
		-e "s|_OUT-NAME_|${inFilename}|" \
		-e "s/_BLAST-PROG_/${blastProg}/" < ${combTempFile} > ${outFile}

cmd="qsub -W depend=afterok:$pbsID ${outFile}";
echo "$cmd"; echo;
if [ ! -z $DEBUG ] && [ "$DEBUG" -lt 1 ]; then
	$cmd;
else
	echo "DEBUG mode, skip execution: $cmd"
fi

